library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.std_logic_unsigned.all;
 
entity calculate_size is
   port
   (
		clk, reset, start : in std_logic;
      size : in std_logic_vector(12 downto 0); 
		done : out std_logic;
      size1, size2 : out std_logic_vector(11 downto 0)
	);
end entity calculate_size;
 
architecture rtl of calculate_size is
type state_type is (idle, state_one, state_two, finished_state);
signal current_state, next_state : state_type;

signal counter : std_logic_vector(12 downto 0);
signal size1_reg : std_logic_vector(11 downto 0);
signal size2_reg : std_logic_vector(11 downto 0);
signal update_counter, reset_counter, reset_size1, reset_size2, update_size1, update_size2 : std_logic;

begin

process(clk, reset)
	begin
		if(reset = '1') then
			counter <= (OTHERS => '0');
			size1_reg <= (OTHERS => '0');
			size2_reg <= (OTHERS => '0');
			current_state <= idle;
		elsif(rising_edge(clk)) then
			current_state <= next_state;
			if(update_counter = '1') then
				counter <= counter + 1;
			elsif(reset_counter = '1') then
				counter <= (OTHERS => '0');
			end if;
			if(update_size1 = '1') then
				size1_reg <= size1_reg + 1;
			elsif(reset_size1 = '1') then
				size1_reg <= (OTHERS => '0');
			end if;
			if(update_size2 = '1') then
				size2_reg <= size2_reg + 1;
			elsif(reset_size2 = '1') then
				size2_reg <= (OTHERS => '0');
			end if;
		end if;
	end process;
			
	
	process(current_state, size, start, counter)
	begin
	case current_state is
		when idle =>
			update_counter <= '0';
			update_size1 <= '0';
			update_size2 <= '0';
			reset_counter <= '1';
			done <= '0';
			if(start = '1') then
				reset_size1 <= '1';
				reset_size2 <= '1';
				next_state <= state_one;
			else
				next_state <= idle;
				reset_size1 <= '0';
				reset_size2 <= '0';
			end if;
			
		when state_one =>
			reset_counter <= '0';
			reset_size1 <= '0';
			reset_size2 <= '0';
			update_size2 <= '0';
			done <= '0';
			if(counter = size-1) then
				next_state <= finished_state;
				update_counter <= '0';
				update_size1 <= '1';
			else
				next_state <= state_two;
				update_counter <= '1';
				update_size1 <= '1';
			end if;
			
		when state_two =>
			reset_counter <= '0';
			reset_size1 <= '0';
			reset_size2 <= '0';
			update_size1 <= '0';
			done <= '0';
			if(counter = size-1) then
				next_state <= finished_state;
				update_counter <= '0';
				update_size2 <= '1';
			else
				next_state <= state_one;
				update_counter <= '1';
				update_size2 <= '1';
			end if;
		
		when finished_state =>
			done <= '1';
			reset_counter <= '1';
			update_counter <= '0';
			update_size1 <= '0';
			update_size2 <= '0';
			reset_size1 <= '0';
			reset_size2 <= '0';
			next_state <= idle;
		end case;
	end process;
	
	size1 <= size1_reg;
	size2 <= size2_reg;
end architecture rtl;