library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity control_dot_product is
	port(
		clk : in std_logic;
		reset : in std_logic;
		size : in std_logic_vector(12 downto 0);--12 downto 0-------------------------------------------------------------------------------------
		start : in std_logic;
		mode : in std_logic;
		calculate : out std_logic;
		reset_data_reg : out std_logic;
		d_sel1 : out std_logic;
		d_sel2 : out std_logic_vector(2 downto 0);
		d_sel3 : out std_logic;
		done_rr : out std_logic;
		done_dq : out std_logic
	);
end control_dot_product;
	
architecture rtl of control_dot_product is

	type state_type is (idle, calculate_state, wait_state, wait1_state, acc1_state, acc2_state, acc3_state, acc4_state, wait2_state);
	--signal
   signal current_state, next_state : state_type;
	signal teller: std_logic_vector(12 downto 0);-- := (OTHERS => '0');--12 downto 0----------------------------------------------------------
	signal teller2 : std_logic_vector(12 downto 0);
	signal update_teller, reset_teller, update_teller2, reset_teller2 : std_logic;
	begin
		process(clk, reset)
		begin
			if(reset = '1') then
				current_state <= idle;
				teller <= (OTHERS => '0');
				teller2 <= (OTHERS => '0');
			elsif(rising_edge(clk)) then
				current_state <= next_state;
				if(update_teller = '1') then
					teller <= teller + '1';
				end if;
				if(reset_teller = '1') then
					teller <= (OTHERS => '0');
				end if;
				if(update_teller2 = '1') then
					teller2 <= teller2 + '1';
				elsif(reset_teller2 = '1') then
					teller2 <= (OTHERS => '0');
				end if;
			end if;
		end process;
				
		process(current_state, start, mode, size, teller, teller2)
		begin
			case current_state is
				when idle =>
					d_sel1 <= '0';
					d_sel2 <= "101";--0
					d_sel3 <= '0';
					reset_data_reg <= '1';
					calculate <= '0';
					done_rr <= '0';
					done_dq <= '0';
					reset_teller <= '1';
					update_teller <= '0';
					reset_teller2 <= '1';
					update_teller2 <= '0';
					if(start = '1') then
						next_state <= wait_state;
					else
						next_state <= idle;
					end if;
					
				when wait_state =>
					d_sel1 <= '0';
					d_sel2 <= "111";
					d_sel3 <= '1';
					reset_data_reg <= '1';
					calculate <= '0';
					done_rr <= '0';
					done_dq <= '0';
					reset_teller <= '0';
					update_teller <= '0';
					reset_teller2 <= '0';
					update_teller2 <= '0';
					next_state <= calculate_state;
				
				when calculate_state =>
					d_sel1 <= '0';
					d_sel2 <= "111";
					d_sel3 <= '1';
					reset_data_reg <= '1';
					done_rr <= '0';
					done_dq <= '0';
					reset_teller2 <= '0';
					update_teller2 <= '0';
					if(teller = size) then
						calculate <= '0';
						reset_teller <= '1';
						update_teller <= '0';
						next_state <= wait1_state;
					else
						next_state <= calculate_state;
						calculate <= '1';
						reset_teller <= '0';
						update_teller <= '1';
					end if;
				
					when wait1_state =>
					calculate <= '0';
					done_rr <= '0';
					done_dq <= '0';
					update_teller <= '0';
					reset_teller <= '0';
					d_sel3 <= '1';
					if(teller2 > "0000000001001") then-- 4 3 9
						d_sel2 <= "000";
						d_sel1 <= '1';
						reset_data_reg <= '0';
						reset_teller2 <= '1';
						update_teller2 <= '0';
						next_state <= acc1_state;
					else
						next_state <= wait1_state;
						reset_data_reg <= '1';
						d_sel2 <= "111";
						d_sel1 <= '0';
						reset_teller2 <= '0';
						update_teller2 <= '1';
					end if;
				
				when acc1_state =>
					d_sel1 <= '1';
					d_sel3 <= '1';
					calculate <= '0';
					done_rr <= '0';
					done_dq <= '0';
					update_teller <= '0';
					reset_teller <= '0';
					
					if(teller2 > "0000000001100") then --6 5 12
						d_sel2 <= "001";
						reset_data_reg <= '1';
						update_teller2 <= '0';
						reset_teller2 <= '1';
						next_state <= acc2_state;
					else
						next_state <= acc1_state;
						update_teller2 <= '1';
						reset_teller2 <= '0';
						d_sel2 <= "000";
						reset_data_reg <= '0';
					end if;
				
				when acc2_state =>
					d_sel1 <= '1';
					d_sel3 <= '1';
					calculate <= '0';
					done_rr <= '0';
					done_dq <= '0';
					update_teller <= '0';
					reset_teller <= '0';
					if(teller2 > "0000000001100") then --6 5 12
						d_sel2 <= "010";
						update_teller2 <= '0';
					   reset_teller2 <= '1';
						reset_data_reg <= '1';
						next_state <= acc3_state;
					else
						next_state <= acc2_state;
						update_teller2 <= '1';
					   reset_teller2 <= '0';
						d_sel2 <= "001";
						reset_data_reg <= '0';
					end if;
					
				when acc3_state =>
					d_sel1 <= '1';
					d_sel3 <= '1';
					calculate <= '0';
					done_rr <= '0';
					done_dq <= '0';
					update_teller <= '0';
					reset_teller <= '0';
					if(teller2 > "0000000001100") then -- 6 5 12
						d_sel2 <= "011";
						reset_data_reg <= '1';
						update_teller2 <= '0';
					   reset_teller2 <= '1';
						next_state <= acc4_state;
					else
						next_state <= acc3_state;
						update_teller2 <= '1';
					   reset_teller2 <= '0';
						reset_data_reg <= '0';
						d_sel2 <= "010";
					end if;
					
				when acc4_state =>
					d_sel1 <= '1';
					d_sel3 <= '1';
					calculate <= '0';
					done_rr <= '0';
					done_dq <= '0';
					update_teller <= '0';
					reset_teller <= '0';
					if(teller2 > "0000000001100") then -- 6 5 12
						d_sel2 <= "100";
						reset_data_reg <= '1';
						update_teller2 <= '0';
					   reset_teller2 <= '1';
						next_state <= wait2_state;
					else
						next_state <= acc4_state;
						update_teller2 <= '1';
					   reset_teller2 <= '0';
						reset_data_reg <= '0';
						d_sel2 <= "011";
					end if;
				
				when wait2_state =>
					d_sel1 <= '0';
					d_sel2 <= "100";
					d_sel3 <= '1';
					calculate <= '0';
					done_rr <= '0';
					done_dq <= '0';
					update_teller <= '0';
					reset_teller <= '0';
					reset_data_reg <= '1';
					if(teller2 > "0000000001011") then -- 5 4 11
						update_teller2 <= '0';
					   reset_teller2 <= '0';
						next_state <= idle;
						if(mode = '1') then
							done_rr <= '1';
						else 
							done_dq <= '1';
						end if;
					else
						next_state <= wait2_state;
						update_teller2 <= '1';
					   reset_teller2 <= '0';
						done_rr <= '0';
						done_dq <= '0';
					end if;
			end case;
		end process;
	end rtl;
--library ieee;
--use ieee.std_logic_1164.all;
--use ieee.std_logic_unsigned.all;
--
--entity control_dot_product is
--	port(
--		clk : in std_logic;
--		reset : in std_logic;
--		size : in std_logic_vector(3 downto 0);--12 downto 0-------------------------------------------------------------------------------------
--		start : in std_logic;
--		mode : in std_logic;
--		calculate : out std_logic;
--		d_sel1 : out std_logic;
--		d_sel2 : out std_logic;
--		d_sel3 : out std_logic;
--		d_sel4 : out std_logic_vector(1 downto 0);
--		d_sel5 : out std_logic;
--		done_rr : out std_logic;
--		done_dq : out std_logic
--	);
--end control_dot_product;
--	
--architecture rtl of control_dot_product is
--	
--	type state_type is (idle, calculate_state, wait_state, wait1_state, acc1_state, acc2_state, acc3_state, wait2_state);
--	--signal
--	signal current_state, next_state : state_type;
--	signal teller: std_logic_vector(3 downto 0);-- := (OTHERS => '0');--12 downto 0----------------------------------------------------------
--	signal teller2 : std_logic_vector(2 downto 0);
--	signal update_teller, reset_teller, update_teller2, reset_teller2 : std_logic;
--	begin
--		process(clk, reset, update_teller, reset_teller)
--		begin
--			if(reset = '1') then
--				current_state <= idle;
--				teller <= (OTHERS => '0');
--				teller2 <= (OTHERS => '0');
--			elsif(rising_edge(clk)) then
--				current_state <= next_state;
--				if(update_teller = '1') then
--					teller <= teller + '1';
--				end if;
--				if(reset_teller = '1') then
--					teller <= (OTHERS => '0');
--				end if;
--				if(update_teller2 = '1') then
--					teller2 <= teller2 + '1';
--				elsif(reset_teller2 = '1') then
--					teller2 <= (OTHERS => '0');
--				end if;
--			end if;
--		end process;
--				
--		process(current_state, start, mode, size, teller, teller2)
--		begin
--			case current_state is
--				when idle =>
--					d_sel1 <= '0';
--					d_sel2 <= '0';
--					d_sel3 <= '0';
--					d_sel4 <= "11";
--					d_sel5 <= '0';
--					calculate <= '0';
--					done_rr <= '0';
--					done_dq <= '0';
--					reset_teller <= '1';
--					update_teller <= '0';
--					reset_teller2 <= '1';
--					update_teller2 <= '0';
--					if(start = '1') then
--						next_state <= wait_state;
--					else
--						next_state <= idle;
--					end if;
--					
--				when wait_state =>
--					d_sel1 <= '0';
--					d_sel2 <= '0';
--					d_sel3 <= '0';
--					d_sel4 <= "11";
--					d_sel5 <= '0';
--					calculate <= '0';
--					done_rr <= '0';
--					done_dq <= '0';
--					reset_teller <= '0';
--					update_teller <= '0';
--					reset_teller2 <= '0';
--					update_teller2 <= '0';
--					next_state <= calculate_state;
--					
--				when calculate_state =>
--					d_sel1 <= '0';
--					d_sel2 <= '0';
--					d_sel3 <= '0';
--					d_sel4 <= "11";
--					d_sel5 <= '1';
--					calculate <= '1';
--					done_rr <= '0';
--					done_dq <= '0';
--					update_teller <= '1';
--					reset_teller <= '0';
--					reset_teller2 <= '0';
--					update_teller2 <= '0';
--					if(teller = size) then
--						calculate <= '0';
--						reset_teller <= '1';
--						update_teller <= '0';
--						next_state <= wait1_state;
--					else
--						next_state <= calculate_state;
--						calculate <= '1';
--						reset_teller <= '0';
--						update_teller <= '1';
--					end if;
--					
--					when wait1_state =>
--					d_sel1 <= '0';
--					d_sel2 <= '0';
--					d_sel3 <= '0';
--					d_sel4 <= "11";
--					d_sel5 <= '1';
--					calculate <= '0';
--					done_rr <= '0';
--					done_dq <= '0';
--					update_teller <= '0';
--					reset_teller <= '0';
--					--reset_teller2 <= '0';
--					--update_teller2 <= '1';
--					if(teller2 = "100") then-- 4
--						d_sel1 <= '1';
--						d_sel4 <= "00";
--						reset_teller2 <= '1';
--						update_teller2 <= '0';
--						next_state <= acc1_state;
--					else
--						next_state <= wait1_state;
--						d_sel1 <= '0';
--						d_sel4 <= "11";
--						reset_teller2 <= '0';
--						update_teller2 <= '1';
--					end if;
--					
--				when acc1_state =>
--					d_sel1 <= '1';
--					d_sel2 <= '0';
--					d_sel3 <= '0';
--					d_sel4 <= "00";
--					d_sel5 <= '1';
--					calculate <= '0';
--					done_rr <= '0';
--					done_dq <= '0';
--					update_teller <= '0';
--					reset_teller <= '0';
--					if(teller2 = "110") then
--						d_sel1 <= '0';
--						d_sel2 <= '1';
--						d_sel4 <= "01";
--						update_teller2 <= '0';
--						reset_teller2 <= '1';
--						next_state <= acc2_state;
--					else
--						next_state <= acc1_state;
--						update_teller2 <= '1';
--						reset_teller2 <= '0';
--						d_sel4 <= "00";
--						d_sel1 <= '1';
--						d_sel2 <= '0';
--					end if;
--					
--				when acc2_state =>
--					d_sel1 <= '0';
--					d_sel2 <= '1';
--					d_sel3 <= '0';
--					d_sel4 <= "01";
--					d_sel5 <= '1';
--					calculate <= '0';
--					done_rr <= '0';
--					done_dq <= '0';
--					update_teller <= '0';
--					reset_teller <= '0';
--					if(teller2 = "110") then
--						d_sel2 <= '0';
--						d_sel3 <= '1';
--						d_sel4 <= "10";
--						update_teller2 <= '0';
--					   reset_teller2 <= '1';
--						next_state <= acc3_state;
--					else
--						next_state <= acc2_state;
--						update_teller2 <= '1';
--					   reset_teller2 <= '0';
--						d_sel2 <= '1';
--						d_sel3 <= '0';
--						d_sel4 <= "01";
--					end if;
--					
--				when acc3_state =>
--					d_sel1 <= '0';
--					d_sel2 <= '0';
--					d_sel3 <= '1';
--					d_sel4 <= "10";
--					d_sel5 <= '1';
--					calculate <= '0';
--					done_rr <= '0';
--					done_dq <= '0';
--					update_teller <= '0';
--					reset_teller <= '0';
--					if(teller2 = "110") then
--						d_sel3 <= '0';
--						d_sel4 <= "11";
--						update_teller2 <= '0';
--					   reset_teller2 <= '1';
--						next_state <= wait2_state;
--					else
--						next_state <= acc3_state;
--						update_teller2 <= '1';
--					   reset_teller2 <= '0';
--						d_sel3 <= '1';
--						d_sel4 <= "10";
--					end if;
--					
--				when wait2_state =>
--					d_sel1 <= '0';
--					d_sel2 <= '0';
--					d_sel3 <= '0';
--					d_sel4 <= "11";
--					d_sel5 <= '1';
--					calculate <= '0';
--					done_rr <= '0';
--					done_dq <= '0';
--					update_teller <= '0';
--					reset_teller <= '0';
--					
--					if(teller2 = "101") then
--						update_teller2 <= '0';
--					   reset_teller2 <= '0';
--						if(mode = '1') then
--							done_rr <= '1';
--						else 
--							done_dq <= '1';
--						end if;
--						
--						next_state <= idle;
--					else
--						next_state <= wait2_state;
--						update_teller2 <= '1';
--					   reset_teller2 <= '0';
--						done_rr <= '0';
--						done_dq <= '0';
--					end if;
--					
--				when others =>
--					next_state <= idle;
--					d_sel1 <= '0';
--					d_sel2 <= '0';
--					d_sel3 <= '0';
--					d_sel4 <= "11";
--					d_sel5 <= '0';
--					update_teller <= '0';
--					reset_teller <= '1';
--					reset_teller2 <= '1';
--					update_teller2 <= '0';
--					calculate <= '0';
--					done_rr <= '0';
--					done_dq <= '0';
--			end case;
--		end process;	
--end rtl;