library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.numeric_std.all;
use ieee.std_logic_unsigned.all;


entity control_read_matrix is 
port
(
	clk, reset, start : in std_logic;
	size        : in std_logic_vector(7 downto 0);
	read_matrix, done : out std_logic
);
end entity control_read_matrix;


architecture rtl of control_read_matrix is

signal counter : std_logic_vector(7 downto 0);
signal update_counter, reset_counter : std_logic;
type state_type is (idle, read_state);
signal current_state, next_state : state_type;
begin

process(clk, reset) 
 begin
	if(reset = '1') then
		current_state <= idle;
		counter <= (OTHERS => '0');
   elsif(rising_edge(clk)) then
		current_state <= next_state;
		if(update_counter = '1') then
			counter <= counter + 1;
		elsif(reset_counter = '1') then
			counter <= (OTHERS => '0');
		end if;
	end if;
 end process;
 
 process(current_state, start, counter, size)
 begin
	case current_state is
		when idle =>
			read_matrix <= '0';
			update_counter <= '0';
			reset_counter <= '1';
			done <= '0';
			if(start = '1') then
				next_state <= read_state;
			else
				next_state <= idle;
			end if;
			
		when read_state =>
		   if(counter = size) then
				next_state <= idle;
				done <= '1';
				read_matrix <= '0';
				update_counter <= '0';
				reset_counter <= '1';
			else
				next_state <= read_state;
				done <= '0';
				read_matrix <= '1';
				update_counter <= '1';
				reset_counter <= '0';
			end if;
		end case;
	end process;
end rtl;