library altera;
use altera.altera_europa_support_lib.all;

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

entity custom_driver is 
        port (
              -- inputs:
                 signal clk : IN STD_LOGIC;
                -- signal local_rdata : IN STD_LOGIC_VECTOR (63 DOWNTO 0);
                 signal local_rdata_valid : IN STD_LOGIC;
                 signal local_ready : IN STD_LOGIC;
                 signal reset_n : IN STD_LOGIC;
					  signal local_wdata_req : IN STD_LOGIC;
					  signal jtag_fifo_empty : IN STD_LOGIC;
					  signal fifo1_full : IN STD_LOGIC;
					  signal fifo2_full : IN STD_LOGIC;
					  signal b_fifo_full : IN STD_LOGIC;
					  signal size : IN STD_LOGIC_VECTOR(12 DOWNTO 0);
					  signal done : IN STD_LOGIC;
					  --signal start : IN STD_LOGIC;

              -- outputs:
                 signal local_bank_addr : OUT STD_LOGIC_VECTOR (2 DOWNTO 0);
                 signal local_be : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
                 signal local_col_addr : OUT STD_LOGIC_VECTOR (9 DOWNTO 0);
                 signal local_cs_addr : OUT STD_LOGIC;
                 signal local_read_req : OUT STD_LOGIC;
                 signal local_row_addr : OUT STD_LOGIC_VECTOR (12 DOWNTO 0);
                 signal local_size : OUT STD_LOGIC_VECTOR(1 downto 0);
                 --signal local_wdata : OUT STD_LOGIC_VECTOR (63 DOWNTO 0);
                 signal local_write_req : OUT STD_LOGIC;
					  signal fifo1_write : OUT STD_LOGIC;
					  signal fifo2_write : OUT STD_LOGIC;
					  signal wr_b : OUT STD_LOGIC;
					  signal jtag_fifo_read : OUT STD_LOGIC;
					  --signal done            : OUT STD_LOGIC;
					  signal items_in_fifo1 : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
					  signal items_in_fifo2 : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
					  signal write_jtag_fifo : OUT STD_LOGIC;
					  signal sel_control_mux : OUT STD_LOGIC;
					  signal sel_data_mux : OUT STD_LOGIC;
					  signal ferdig : OUT STD_LOGIC;
					  signal sel_control_mux2 : OUT STD_LOGIC;
					  signal store_size : OUT STD_LOGIC;
					  signal result_fifo_empty : IN STD_LOGIC;
					  signal result_fifo_read : OUT STD_LOGIC;
					  signal start : IN STD_LOGIC
              );
attribute ALTERA_ATTRIBUTE : string;
attribute ALTERA_ATTRIBUTE of custom_driver : entity is "MESSAGE_DISABLE=14130;MESSAGE_DISABLE=14110";
end entity custom_driver;


architecture rtl of custom_driver is
--signals
signal teller1, teller2 : std_logic_vector(63 downto 0);
signal teller3 : std_logic_vector(2 downto 0);
signal teller4 : std_logic_vector(12 downto 0);
signal teller1_update, teller1_reset, teller2_update, teller2_reset, teller3_update, teller3_reset, teller4_update, teller4_reset : std_logic;
signal MAX_ROW : std_logic_vector(12 downto 0);
  --Change MAX_COL to test more or lesser column address in test_seq_addr_mode, maximum value is 2^(column bits) - (LOCAL_BURST_LEN_s * dwidth_ratio (aka half-rate (4) or full-rate (2))), while minimum value is 0 for Half rate and (LOCAL_BURST_LEN_s * dwidth_ratio) for Full rate
signal MAX_COL_WRITE : std_logic_vector(9 downto 0);
signal MAX_COL_READ : std_logic_vector(9 downto 0);
  --Decrease MAX_BANK to test lesser bank address, minimum value is 0
signal MAX_BANK : std_logic_vector(2 downto 0);

signal bank_addr : std_logic_vector(2 downto 0);
signal row_addr : std_logic_vector(12 downto 0);
signal col_addr : std_logic_vector(9 downto 0);
signal reset_address : std_logic;
signal half_size : std_logic_vector(25 downto 0);
signal dobbel_size : std_logic_vector(25 downto 0);
signal read_req, write_req, wdata_req, wait_bit : std_logic;
--signal size : std_logic_vector(3 downto 0);
--signal wdata : std_logic_vector(63 downto 0);
--states
type state_type is (idle, size_state, store_size_state, read_req_to_b, write_to_b, write_to_ddr_state, reset_state2, reset_state, read_ddr_state, wait1_state, wait2_state, write_result_to_jtag_fifo, finish_state);-- wait1_state, wait2_state, write_result_to_jtag_fifo,
signal current_state, next_state : state_type;


begin

--local_size <= "01";--eller "10"
local_be <= (OTHERS => '1');
MAX_ROW <= "1111111111111";--"0000000000011"
MAX_COL_WRITE <= "1111111100";--"0000010000"
MAX_COL_READ <=  "1111111000";
MAX_BANK <= "111";--"001"
local_cs_addr <= '0';
local_bank_addr <= bank_addr;
local_col_addr <= col_addr;
local_row_addr <= row_addr;
local_read_req <= read_req;
local_write_req <= write_req;
dobbel_size <= size*size;
--half_size <= dobbel_size srl 1;
half_size <= to_stdlogicvector(to_bitvector(dobbel_size) srl 1);
--local_wdata <= wdata;

process(clk, reset_n) 
begin
if(reset_n = '0') then
	teller1 <= (OTHERS => '0');
	teller2 <= (OTHERS => '0');
	teller3 <= (OTHERS => '0');
	teller4 <= (OTHERS => '0');
	current_state <= idle;
elsif(rising_edge(clk)) then
	current_state <= next_state;
	if(teller1_update = '1') then
		teller1 <= teller1 + 1;
	elsif(teller1_reset = '1') then
		teller1 <= (OTHERS => '0');
	end if;
	if(teller2_update = '1') then
		teller2 <= teller2 + 1;
	elsif(teller2_reset = '1') then
		teller2 <= (OTHERS => '0');
	end if;
	if(teller3_update = '1') then
		teller3 <= teller3 + 1;
	elsif(teller3_reset = '1') then
		teller3 <= (OTHERS => '0');
	end if;
	if(teller4_update = '1') then
		teller4 <= teller4 + 1;
	elsif(teller4_reset = '1') then
		teller4 <= (OTHERS => '0');
	end if;
end if;
end process;

process(current_state, half_size, teller1, teller2, local_ready, local_rdata_valid, local_wdata_req, jtag_fifo_empty, fifo1_full, fifo2_full, b_fifo_full, size, read_req, items_in_fifo1, items_in_fifo2, teller3, teller4, done, result_fifo_empty, start)
begin
case current_state is

when idle =>
	read_req <= '0';
	write_req <= '0';
	--wait_bit <= '0';
	jtag_fifo_read <= '0';
	fifo1_write <= '0';
	fifo2_write <= '0';
	reset_address <= '1';
	teller1_update <= '0';
	teller1_reset <= '1';
	teller2_update <= '0';
	teller2_reset <= '1';
	teller3_update <= '0';
	teller3_reset <= '1';
	teller4_update <= '0';
	teller4_reset <= '1';
	wr_b <= '0';
	write_jtag_fifo <= '0';
	sel_control_mux <= '0';
	sel_control_mux2 <= '0';
	sel_data_mux <= '0';
	ferdig <= '0';
	store_size <= '0';
	result_fifo_read <= '0';
	local_size <= "01";
	--done <= '0';
	if(local_ready = '1') then
		next_state <= size_state;--read_req_to_b;--size_state;
	else
		next_state <= idle;
	end if;
	
when size_state =>
	read_req <= '0';
	write_req <= '0';
	fifo1_write <= '0';
	fifo2_write <= '0';
	reset_address <= '1';
	teller1_update <= '0';
	teller1_reset <= '1';
	teller2_update <= '0';
	teller2_reset <= '1';
	teller3_update <= '0';
	teller3_reset <= '1';
	teller4_update <= '0';
	teller4_reset <= '1';
	wr_b <= '0';
	write_jtag_fifo <= '0';
	sel_control_mux <= '0';
	sel_control_mux2 <= '0';
	sel_data_mux <= '0';
	ferdig <= '0';
	store_size <= '0';
	local_size <= "01";
   result_fifo_read <= '0';
	if(jtag_fifo_empty = '0') then
		jtag_fifo_read <= '1';
		next_state <= store_size_state;
	else
		jtag_fifo_read <= '0';
		next_state <= size_state;
	end if;
	
when store_size_state =>
	read_req <= '0';
	write_req <= '0';
	jtag_fifo_read <= '0';
	fifo1_write <= '0';
	fifo2_write <= '0';
	reset_address <= '1';
	teller1_update <= '0';
	teller1_reset <= '1';
	teller2_update <= '0';
	teller2_reset <= '1';
	teller3_update <= '0';
	teller3_reset <= '1';
	teller4_update <= '0';
	teller4_reset <= '1';
	wr_b <= '0';
	write_jtag_fifo <= '0';
	sel_control_mux <= '0';
	sel_control_mux2 <= '0';
	sel_data_mux <= '0';
	ferdig <= '0';
	store_size <= '1';
   result_fifo_read <= '0';
	local_size <= "01";
	next_state <= read_req_to_b;
	
	
when read_req_to_b =>
	read_req <= '0';
	write_req <= '0';
	--wait_bit <= '0';
	fifo1_write <= '0';
	fifo2_write <= '0';
	reset_address <= '1';
	teller1_update <= '0';
	teller1_reset <= '1';
	teller2_update <= '0';
	teller2_reset <= '0';
	teller3_update <= '0';
	teller3_reset <= '1';
	teller4_update <= '0';
	teller4_reset <= '1';
	wr_b <= '0';
	write_jtag_fifo <= '0';
	sel_control_mux <= '0';
	sel_control_mux2 <= '0';
	sel_data_mux <= '0';
	ferdig <= '0';
	store_size <= '0';
	result_fifo_read <= '0';
	local_size <= "01";
	
	if(jtag_fifo_empty = '0') then--jtag_fifo_empty = '0' AND b_fifo_full = '0'
		jtag_fifo_read <= '1';
		next_state <= write_to_b;
	else
		jtag_fifo_read <= '0';
		next_state <= read_req_to_b;
	end if;
	
	
	
when write_to_b =>
	read_req <= '0';
	write_req <= '0';
	--wait_bit <= '0';
	jtag_fifo_read <= '0';
	fifo1_write <= '0';
	fifo2_write <= '0';
	reset_address <= '0';
	teller1_update <= '0';
	teller1_reset <= '1';
	--teller2_update <= '0';
	--teller2_reset <= '1';
	teller3_update <= '0';
	teller3_reset <= '1';
	teller4_update <= '0';
	teller4_reset <= '1';
	wr_b <= '1';
	write_jtag_fifo <= '0';
	sel_control_mux <= '0';
	sel_control_mux2 <= '0';
	sel_data_mux <= '0';
	ferdig <= '0';
	store_size <= '0';
	result_fifo_read <= '0';
	local_size <= "01";
	
	if(teller2 = size-1) then --size-1 03------
		next_state <= write_to_ddr_state;
		teller2_reset <= '1';
		teller2_update <= '0';
	else
		next_state <= read_req_to_b;
		teller2_reset <= '0';
		teller2_update <= '1';
	end if;
	
--when write_req_state =>
--	read_req <= '0';
--	reset_address <= '0';
--	teller4_update <= '0';
--	teller4_reset <= '0';
--	teller3_update <= '0';
--	teller3_reset <= '0';
--	teller2_update <= '0';
--	teller2_reset <= '0';
--	teller1_update <= '0';
--	teller1_reset <= '0';
--	fifo1_write <= '0';
--	fifo2_write <= '0';
--	jtag_fifo_read <= '0';
--	wr_b <= '0';
--	write_jtag_fifo <= '0';
--	sel_control_mux <= '0';
--	sel_control_mux2 <= '0';
--	sel_data_mux <= '0';
--	ferdig <= '0';
--	store_size <= '0';
--	result_fifo_read <= '0';
--	--done <= '0';
--	
--	if(jtag_fifo_empty = '0') then--jtag_fifo_empty = '0' AND local_ready = '1'
--		write_req <= '1';
--		next_state <= write_to_ddr_state;
--	else
--		write_req <= '0';
--		next_state <= write_req_state;
--	end if;
	
when write_to_ddr_state =>	
	read_req <= '0';
	--write_req <= '0';------------------------------
	reset_address <= '0';
	teller2_update <= '0';
	teller2_reset <= '0';
	--teller3_update <= '0';
	--teller3_reset <= '0';
	teller4_update <= '0';
	teller4_reset <= '1';
	fifo1_write <= '0';
	fifo2_write <= '0';
	wr_b <= '0';
	write_jtag_fifo <= '0';
	sel_control_mux <= '0';
	sel_control_mux2 <= '0';
	sel_data_mux <= '0';
	ferdig <= '0';
	store_size <= '0';
	result_fifo_read <= '0';
	local_size <= "01";
	
	if(jtag_fifo_empty = '0' AND teller3 = "000") then
		write_req <= '1';
		teller3_update <= '1';
		--teller3_reset <= '0';
		--next_state <= write_to_ddr_state;
	else
		write_req <= '0';
		teller3_update <= '0';
		
		--next_state <= write_req_state;
	end if;
	--done <= '0';
	
	if(local_wdata_req = '1') then
		--teller1_update <= '1';
		jtag_fifo_read <= '1';
		teller3_reset <= '1';
		if(teller1 = size*size-1) then--size*size-1 F-------------------------
			next_state <= reset_state2;
			teller1_update <= '0';
			teller1_reset <= '1';
		else
			next_state <= write_to_ddr_state;--write_req_state;
			teller1_update <= '1';
			teller1_reset <= '0';
		end if;
	else	
		next_state <= write_to_ddr_state;
		jtag_fifo_read <= '0';
		teller1_reset <= '0';
		teller1_update <= '0';
		teller3_reset <= '0';
	end if;
	
--	when wait_on_empty_state =>
--		read_req <= '0';
--		write_req <= '0';
--		wait_bit <= '0';
--		jtag_fifo_read <= '0';
--		fifo1_write <= '0';
--		fifo2_write <= '0';
--		reset_address <= '0';
--		teller1_update <= '0';
--		teller1_reset <= '0';
--		teller2_update <= '0';
--		teller2_reset <= '0';
--		teller3_update <= '0';
--		teller3_reset <= '0';
--		teller4_update <= '0';
--		teller4_reset <= '0';
--		wr_b <= '0';
--		write_jtag_fifo <= '0';
--		sel_control_mux <= '0';
--		sel_control_mux2 <= '0';
--		sel_data_mux <= '0';
--		ferdig <= '0';
--		store_size <= '0';
--		result_fifo_read <= '0';
--		next_state <= write_to_ddr_state;
	when reset_state2 =>
		read_req <= '0';
		write_req <= '0';
		--wait_bit <= '0';
		jtag_fifo_read <= '0';
		fifo1_write <= '0';
		fifo2_write <= '0';
		reset_address <= '1';
		teller1_update <= '0';
		teller1_reset <= '1';
		teller2_update <= '0';
		teller2_reset <= '1';
		teller3_update <= '0';
		teller3_reset <= '1';
		teller4_update <= '0';
	   teller4_reset <= '1';
		wr_b <= '0';
		write_jtag_fifo <= '0';
	   sel_control_mux <= '0';
		sel_control_mux2 <= '0';
	   sel_data_mux <= '0';
		ferdig <= '0';
		store_size <= '0';
		result_fifo_read <= '0';
		--done <= '0';
		local_size <= "01";
		next_state <= reset_state;
	
	when reset_state =>
		read_req <= '0';
		write_req <= '0';
		--wait_bit <= '0';
		jtag_fifo_read <= '0';
		fifo1_write <= '0';
		fifo2_write <= '0';
		reset_address <= '1';
		teller1_update <= '0';
		teller1_reset <= '1';
		teller2_update <= '0';
		teller2_reset <= '1';
		teller3_update <= '0';
		teller3_reset <= '1';
		teller4_update <= '0';
	   teller4_reset <= '1';
		wr_b <= '0';
		write_jtag_fifo <= '0';
	   sel_control_mux <= '0';
		sel_control_mux2 <= '0';
	   sel_data_mux <= '0';
		ferdig <= '0';
		store_size <= '0';
		result_fifo_read <= '0';
		--done <= '0';
		local_size <= "10";
		next_state <= read_ddr_state;
	
	when read_ddr_state =>
		write_req <= '0';
		--wait_bit <= '0';
		jtag_fifo_read <= '0';
		reset_address <= '0';
		teller1_reset <= '0';
		teller2_reset <= '0';
		wr_b <= '0';
		write_jtag_fifo <= '0';
	   sel_control_mux <= '0';
		sel_control_mux2 <= '0';
	   sel_data_mux <= '0';
		ferdig <= '0';
		store_size <= '0';
		result_fifo_read <= '0';
		local_size <= "10";
		--done <= '0';
		
		if(items_in_fifo1 < "01000000" and items_in_fifo2 < "01000000") then
			--if(teller2 < size*size) then
			if(teller2 < x"00000000000000C8") then--x"0000000000000032" size*size
				read_req <= '1';
			else
				read_req <= '0';
			end if;
		else
			read_req <= '0';
		end if;
	
		if(local_ready = '1' and read_req = '1') then
			teller2_update <= '1';
		else
			teller2_update <= '0';
		end if;
		
		if(local_rdata_valid = '1') then
			teller1_update <= '1';
--			 if(teller1 = size*size) then --size*size--------10
--				if(done = '1') then
--					next_state <= wait1_state;--finish_state;--reset2_state;
--					reset_address <= '1';
--					teller1_reset <= '1';
--					teller1_update <= '0';
--					teller2_reset <= '1';
--				else
--					next_state <= read_ddr_state;--reset_state;
--					reset_address <= '1';
--					teller1_reset <= '1';
--					teller1_update <= '0';
--					teller2_reset <= '1';
--				end if;
--			else
--				if(done = '1') then
--					next_state <= wait1_state;--finish_state;--reset2_state;
--					reset_address <= '1';
--					teller1_reset <= '1';
--					teller1_update <= '0';
--					teller2_reset <= '1';
--				else
--					next_state <= read_ddr_state;
--					reset_address <= '0';
--					teller1_reset <= '0';
--					teller1_update <= '1';
--					teller2_reset <= '0';
--				end if;
--			end if;
			
			if(teller3 = "000") then
				fifo1_write <= '1';
				fifo2_write <= '0';
				if(teller4 = size -1) then
					teller3_update <= '0';
					teller3_reset <= '1';
					teller4_reset <= '1';
					teller4_update <= '0';
				else
					teller3_update <= '1';
					teller3_reset <= '0';
					teller4_reset <= '0';
					teller4_update <= '1';
				end if;
			elsif(teller3 = "001") then
				fifo2_write <= '1';
				fifo1_write <= '0';
				teller3_reset <= '1';
				teller3_update <= '0';
				teller4_update <= '1';
				teller4_reset <= '0';
			else
				fifo2_write <= '0';
				fifo1_write <= '0';
				teller3_reset <= '0';
				teller3_update <= '0';
				teller4_update <= '0';
				teller4_reset <= '0';
			end if;
		else
			teller1_update <= '0';
			--teller1_reset <= '0';
			--teller2_reset <= '0';--
			teller3_update <= '0';
			teller3_reset <= '0';
			teller4_update <= '0';
			teller4_reset <= '0';
			--reset_address <= '0';--
			--next_state <= read_ddr_state;--
			fifo2_write <= '0';
			fifo1_write <= '0';
		end if;
		
	   if(teller1 = size*size-1) then --size*size--------10
			if(done = '1') then
				next_state <= wait1_state;--finish_state;--reset2_state;
				--reset_address <= '1';
				--teller1_reset <= '1';
				--teller2_reset <= '1';
			else
				next_state <= reset_state;
				--reset_address <= '1';
				--teller1_reset <= '1';
				--teller2_reset <= '1';
			end if;
		else
			if(done = '1') then
				next_state <= wait1_state;--finish_state;--reset2_state;
				--reset_address <= '1';
				--teller1_reset <= '1';
				--teller2_reset <= '1';
			else
				next_state <= read_ddr_state;
				--reset_address <= '0';
				--teller1_reset <= '0';
				--teller2_reset <= '0';
			end if;
		end if;
		
	when wait1_state =>
		read_req <= '0';
		write_req <= '0';
		--wait_bit <= '0';
		jtag_fifo_read <= '0';
		fifo1_write <= '0';
		fifo2_write <= '0';
		reset_address <= '1';
		teller1_update <= '0';
		teller1_reset <= '1';
		teller2_update <= '0';
		teller2_reset <= '1';
		teller3_update <= '0';
		teller3_reset <= '1';
		teller4_update <= '0';
	   teller4_reset <= '1';
		wr_b <= '0';
		write_jtag_fifo <= '0';
		sel_control_mux <= '0';
		sel_data_mux <= '0';
		sel_control_mux2 <= '0';
		ferdig <= '0';
		store_size <= '0';
		result_fifo_read <= '0';
		local_size <= "01";
		next_state <= wait2_state;--write_result_to_jtag_fifo;
		
	when wait2_state =>
		read_req <= '0';
		write_req <= '0';
		--wait_bit <= '0';
		jtag_fifo_read <= '0';
		fifo1_write <= '0';
		fifo2_write <= '0';
		reset_address <= '1';
		--teller1_update <= '0';
		teller1_reset <= '0';
		teller2_update <= '0';
		teller2_reset <= '1';
		teller3_update <= '0';
		teller3_reset <= '1';
		teller4_update <= '0';
	   teller4_reset <= '1';
		wr_b <= '0';
		write_jtag_fifo <= '0';
		sel_control_mux <= '0';
		sel_control_mux2 <= '0';
		sel_data_mux <= '0';
		ferdig <= '0';
		store_size <= '0';
		local_size <= "01";
		--result_fifo_read <= '0';
		if(result_fifo_empty = '0') then
			result_fifo_read <= '1';
			teller1_update <= '1';
			next_state <=  write_result_to_jtag_fifo;
		else
			result_fifo_read <= '0';
			teller1_update <= '0';
			next_state <=  wait2_state;
		end if;
		--next_state <= write_result_to_jtag_fifo;
		
	when write_result_to_jtag_fifo =>
		read_req <= '0';
		write_req <= '0';
		--wait_bit <= '0';
		jtag_fifo_read <= '0';
		fifo1_write <= '0';
		fifo2_write <= '0';
		reset_address <= '1';
		teller1_update <= '0';
		--teller1_reset <= '0';
		teller2_update <= '0';
		teller2_reset <= '1';
		teller3_update <= '0';
		teller3_reset <= '1';
		teller4_update <= '0';
	   teller4_reset <= '1';
		wr_b <= '0';
		write_jtag_fifo <= '1';
		sel_control_mux <= '1';
		sel_control_mux2 <= '0';
		sel_data_mux <= '1';
		ferdig <= '0';
		store_size <= '0';
		result_fifo_read <= '0';
		local_size <= "01";
		--next_state <= finish_state;
		if(teller1 = size) then--x"0000000000000003"size-1
			next_state <= finish_state;
			teller1_reset <= '1';
		else
			next_state <= wait2_state;--write_result_to_jtag_fifo;
			teller1_reset <= '0';
		end if;
--		
	when finish_state =>
		read_req <= '0';
		write_req <= '0';
		--wait_bit <= '0';
		reset_address <= '1';
		teller1_update <= '0';
		teller1_reset <= '1';
		teller2_update <= '0';
		teller2_reset <= '1';
		teller3_update <= '0';
		teller3_reset <= '1';
		teller4_update <= '0';
	   teller4_reset <= '1';
		fifo1_write <= '0';
		fifo2_write <= '0';
		jtag_fifo_read <= '0';
		wr_b <= '0';
		write_jtag_fifo <= '0';
		sel_control_mux <= '0';
		sel_control_mux2 <= '1';
		sel_data_mux <= '0';
		ferdig <= '1';
		store_size <= '0';
		result_fifo_read <= '0';
		local_size <= "01";
		next_state <= finish_state;
		
	end case;
end process;

--process(clk, reset_address)
--begin
--if(reset_address = '1') then
--	col_addr <= (OTHERS => '0');
--	row_addr <= (OTHERS => '0');
--	bank_addr <= (OTHERS => '0');
--elsif(rising_edge(clk)) then
--	if ((local_ready = '1' AND write_req = '1') OR (local_ready = '1' AND read_req = '1')) then
--        if (col_addr >= MAX_COL) then 
--          col_addr <= (OTHERS=>'0');
--          if (row_addr = MAX_ROW) then 
--            row_addr <= (OTHERS=>'0');
--            if (bank_addr = MAX_BANK) then 
--              bank_addr <= (OTHERS=>'0');
--            else
--              bank_addr <= bank_addr + 1; 
--            end if;
--          else
--            row_addr <= row_addr + 1; 
--          end if;
--        else
--          col_addr <= col_addr + 4;--eller 8
--        end if;
--	 end if;
--end if;
--end process;
process(clk, reset_address)
begin
if(reset_address = '1') then
	col_addr <= (OTHERS => '0');
	row_addr <= (OTHERS => '0');
	bank_addr <= (OTHERS => '0');
elsif(rising_edge(clk)) then
	if (local_ready = '1' AND write_req = '1') then
        if (col_addr >= MAX_COL_WRITE) then 
          col_addr <= (OTHERS=>'0');
          if (row_addr = MAX_ROW) then 
            row_addr <= (OTHERS=>'0');
            if (bank_addr = MAX_BANK) then 
              bank_addr <= (OTHERS=>'0');
            else
              bank_addr <= bank_addr + 1; 
            end if;
          else
            row_addr <= row_addr + 1; 
          end if;
        else
          col_addr <= col_addr + 4;--eller 8
        end if;
	 elsif (local_ready = '1' AND read_req = '1') then
        if (col_addr >= MAX_COL_READ) then 
          col_addr <= (OTHERS=>'0');
          if (row_addr = MAX_ROW) then 
            row_addr <= (OTHERS=>'0');
            if (bank_addr = MAX_BANK) then 
              bank_addr <= (OTHERS=>'0');
            else
              bank_addr <= bank_addr + 1; 
            end if;
          else
            row_addr <= row_addr + 1; 
          end if;
        else
          col_addr <= col_addr + 8;--eller 8
        end if;
	 end if;
end if;
end process;
end rtl;
--	 elsif(local_ready = '1' AND read_req = '1') then
--		 if (col_addr >= MAX_COL) then 
--          col_addr <= (OTHERS=>'0');
--          if (row_addr = MAX_ROW) then 
--            row_addr <= (OTHERS=>'0');
--            if (bank_addr = MAX_BANK) then 
--              bank_addr <= (OTHERS=>'0');
--            else
--              bank_addr <= bank_addr + 1; 
--            end if;
--          else
--            row_addr <= row_addr + 1; 
--          end if;
--        else
--          col_addr <= col_addr + 4;--eller 8
--        end if;
--    end if;
--end if;
--end process;

--process(clk, reset_address)
--begin
--if(reset_address = '1') then
--	col_addr <= (OTHERS => '0');
--	row_addr <= (OTHERS => '0');
--	bank_addr <= (OTHERS => '0');
--elsif(rising_edge(clk)) then
--	if ((local_ready = '1' AND write_req = '1') OR (local_ready = '1' AND read_req = '1')) then
--        if (col_addr >= MAX_COL) then 
--          col_addr <= (OTHERS=>'0');
--          if (row_addr = MAX_ROW) then 
--            row_addr <= (OTHERS=>'0');
--            if (bank_addr = MAX_BANK) then 
--              bank_addr <= (OTHERS=>'0');
--            else
--              bank_addr <= bank_addr + 1; 
--            end if;
--          else
--            row_addr <= row_addr + 1; 
--          end if;
--        else
--          col_addr <= col_addr + 4;--eller 8
--        end if;
--    end if;
--end if;
--end process;
	 
--end rtl;	
--when skriv_ut_data_state =>
--	test_complete <= '0';
--	pnf_per_byte <= "00000000";
--	pnf_persist <= '0';
--	test_status <=  "00000000";
--	read_req <= '0';
--	reset_address <= '0';
--	teller2_update <= '0';
--	teller2_reset <= '0';
--	write_req <= '1';
--	teller1_update <= '1';
--	if(local_wdata_req = '1') then
--		if(teller1 > x"0000000000001000") then
--			next_state <= reset_state;
--			teller1_reset <= '1';
--		else
--			next_state <= skriv_ut_data_state;
--			teller1_reset <= '0';
--		end if;
--	else
--		next_state <= skriv_state;
--		teller1_reset <= '0';
--	end if;
--	
--when reset_state =>
--	pnf_per_byte <= "00000000";
--	pnf_persist <= '0';
--	test_status <=  "00000000";
--	test_complete <= '0';
--	read_req <= '0';
--	write_req <= '0';
--	teller2_update <= '0';
--	teller2_reset <= '1';
--	if(teller1 > x"0000000000000100") then
--		reset_address <= '1';
--		teller1_update <= '0';
--		teller1_reset <= '1';
--		next_state <= les_state;
--	else
--		reset_address <= '0';
--		teller1_update <= '1';
--		teller1_reset <= '0';
--		next_state <= reset_state;
--	end if;
--	
--when les_state =>
--	test_complete <= '0';
--	pnf_per_byte <= "00000000";
--	test_status <=  "00000000";
--	write_req <= '0';
--	reset_address <= '0';
--	teller1_reset <= '0';
--	teller2_update <= '0';
--	teller2_reset <= '0';
--	read_req <= '1';
--	
--	if(local_rdata_valid = '1') then
--		teller1_update <= '1';
--		if(local_rdata = teller1) then 
--			pnf_persist <= '0';--RETT
--		else
--			pnf_persist <= '1';--FEIL
--		end if;
--		if(teller1 > x"00000000000000FF") then
--			next_state <= finish_state;
--		else
--			next_state <= les_state;
--		end if;
--	else	
--		pnf_persist <= '0';
--		teller1_update <= '0';
--		next_state <= les_state;
--	end if;
	
	
		
--	when pause_state =>
--		pnf_per_byte <= "00000000";
--		pnf_persist <= '0';
--		test_status <=  "00000000";
--		test_complete <= '0';
--		read_req <= '0';
--		write_req <= '0';
--		reset_address <= '1';
--		teller2_update <= '0';
--		teller2_reset <= '1';
--		if(teller1 > x"0000000000000010") then
--			next_state <= idle;
--			teller1_update <= '0';
--			teller1_reset <= '1';
--		else
--			next_state <= pause_state;
--			teller1_update <= '1';
--			teller1_reset <= '0';
--		end if;
		
--end case;
--end process;




--library altera;
--use altera.altera_europa_support_lib.all;
--
--library ieee;
--use ieee.std_logic_1164.all;
--use ieee.std_logic_arith.all;
--use ieee.std_logic_unsigned.all;
--
--entity custom_driver is 
--        port (
--              -- inputs:
--                 signal clk : IN STD_LOGIC;
--                 signal local_rdata : IN STD_LOGIC_VECTOR (63 DOWNTO 0);
--                 signal local_rdata_valid : IN STD_LOGIC;
--                 signal local_ready : IN STD_LOGIC;
--                 signal reset_n : IN STD_LOGIC;
--					  signal local_wdata_req : IN STD_LOGIC;
--					 -- signal start : IN STD_LOGIC;
--
--              -- outputs:
--                 signal local_bank_addr : OUT STD_LOGIC_VECTOR (2 DOWNTO 0);
--                 signal local_be : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
--                 signal local_col_addr : OUT STD_LOGIC_VECTOR (9 DOWNTO 0);
--                 signal local_cs_addr : OUT STD_LOGIC;
--                 signal local_read_req : OUT STD_LOGIC;
--                 signal local_row_addr : OUT STD_LOGIC_VECTOR (12 DOWNTO 0);
--                 signal local_size : OUT STD_LOGIC_VECTOR(1 downto 0);
--                 signal local_wdata : OUT STD_LOGIC_VECTOR (63 DOWNTO 0);
--                 signal local_write_req : OUT STD_LOGIC;
--                 signal pnf_per_byte : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
--                 signal pnf_persist : OUT STD_LOGIC;
--                 signal test_complete : OUT STD_LOGIC;
--                 signal test_status : OUT STD_LOGIC_VECTOR (7 DOWNTO 0)
--              );
--attribute ALTERA_ATTRIBUTE : string;
--attribute ALTERA_ATTRIBUTE of custom_driver : entity is "MESSAGE_DISABLE=14130;MESSAGE_DISABLE=14110";
--end entity custom_driver;
--
--
--architecture rtl of custom_driver is
----signals
--signal teller1, teller2 : std_logic_vector(63 downto 0);
--
--signal teller1_update, teller1_reset, teller2_update, teller2_reset : std_logic;
--signal MAX_ROW : std_logic_vector(12 downto 0);
--  --Change MAX_COL to test more or lesser column address in test_seq_addr_mode, maximum value is 2^(column bits) - (LOCAL_BURST_LEN_s * dwidth_ratio (aka half-rate (4) or full-rate (2))), while minimum value is 0 for Half rate and (LOCAL_BURST_LEN_s * dwidth_ratio) for Full rate
--signal MAX_COL : std_logic_vector(9 downto 0);
--  --Decrease MAX_BANK to test lesser bank address, minimum value is 0
--signal MAX_BANK : std_logic_vector(2 downto 0);
--
--signal bank_addr : std_logic_vector(2 downto 0);
--signal row_addr : std_logic_vector(12 downto 0);
--signal col_addr : std_logic_vector(9 downto 0);
--signal reset_address : std_logic;
--signal read_req, write_req, wdata_req : std_logic;
--signal wdata : std_logic_vector(63 downto 0);
----states
--type state_type is (idle, skriv_state, skriv_ut_data_state, reset_state, les_state, finish_state, pause_state);
--signal current_state, next_state : state_type;
--
--
--begin
--
--local_size <= "10";--eller "01"
--local_be <= (OTHERS => '1');
--MAX_ROW <= "0000000001111";--"0000000000011"
--MAX_COL <= "1000000000";--"0000010000"
--MAX_BANK <= "111";--"001"
--local_cs_addr <= '0';
--local_bank_addr <= bank_addr;
--local_col_addr <= col_addr;
--local_row_addr <= row_addr;
--local_read_req <= read_req;
--local_write_req <= write_req;
--local_wdata <= wdata;
--
--process(clk, reset_n) 
--begin
--if(reset_n = '0') then
--	teller1 <= (OTHERS => '0');
--	teller2 <= (OTHERS => '0');
--	current_state <= idle;
--elsif(rising_edge(clk)) then
--	current_state <= next_state;
--	if(teller1_update = '1') then
--		teller1 <= teller1 + 1;
--	elsif(teller1_reset = '1') then
--		teller1 <= (OTHERS => '0');
--	end if;
--	if(teller2_update = '1') then
--		teller2 <= teller2 + 1;
--	elsif(teller2_reset = '1') then
--		teller2 <= (OTHERS => '0');
--	end if;
--end if;
--end process;
--
--process(current_state, teller1, teller2, local_ready, local_rdata_valid, local_rdata, local_wdata_req)
--begin
--case current_state is
--
--when idle =>
--	pnf_per_byte <= "00000000";
--	pnf_persist <= '0';
--	test_status <=  "00000000";
--	test_complete <= '0';
--	read_req <= '0';
--   wdata <= (OTHERS => '0');
--	write_req <= '0';
--	reset_address <= '1';
--	teller1_update <= '0';
--	teller1_reset <= '1';
--	teller2_update <= '0';
--	teller2_reset <= '1';
--	next_state <= skriv_state;
--	
--when skriv_state =>
--	test_complete <= '0';
--	pnf_per_byte <= "00000000";
--	pnf_persist <= '0';
--	test_status <=  "00000000";
--	read_req <= '0';
--	reset_address <= '0';
--	teller2_update <= '0';
--	teller2_reset <= '0';
--	write_req <= '1';
--	wdata <= (OTHERS => '0');
--	teller1_update <= '0';
--	if(local_wdata_req = '1') then
--		if(teller1 > x"0000000000000100") then
--			next_state <= reset_state;
--			teller1_reset <= '1';
--		else
--			next_state <= skriv_ut_data_state;
--			teller1_reset <= '0';
--		end if;
--	else	
--		next_state <= skriv_state;
--		teller1_reset <= '0';
--	end if;
--	
--when skriv_ut_data_state =>
--	test_complete <= '0';
--	pnf_per_byte <= "00000000";
--	pnf_persist <= '0';
--	test_status <=  "00000000";
--	read_req <= '0';
--	reset_address <= '0';
--	teller2_update <= '0';
--	teller2_reset <= '0';
--	write_req <= '1';
--	wdata <= teller1;
--	teller1_update <= '1';
--	if(local_wdata_req = '1') then
--		if(teller1 > x"0000000000001000") then
--			next_state <= reset_state;
--			teller1_reset <= '1';
--		else
--			next_state <= skriv_ut_data_state;
--			teller1_reset <= '0';
--		end if;
--	else
--		next_state <= skriv_state;
--		teller1_reset <= '0';
--	end if;
--	
--when reset_state =>
--	pnf_per_byte <= "00000000";
--	pnf_persist <= '0';
--	test_status <=  "00000000";
--	test_complete <= '0';
--	read_req <= '0';
--   wdata <= (OTHERS => '0');
--	write_req <= '0';
--	teller2_update <= '0';
--	teller2_reset <= '1';
--	if(teller1 > x"0000000000000100") then
--		reset_address <= '1';
--		teller1_update <= '0';
--		teller1_reset <= '1';
--		next_state <= les_state;
--	else
--		reset_address <= '0';
--		teller1_update <= '1';
--		teller1_reset <= '0';
--		next_state <= reset_state;
--	end if;
--	
--when les_state =>
--	test_complete <= '0';
--	pnf_per_byte <= "00000000";
--	test_status <=  "00000000";
--	wdata <= (OTHERS => '0');
--	write_req <= '0';
--	reset_address <= '0';
--	teller1_reset <= '0';
--	teller2_update <= '0';
--	teller2_reset <= '0';
--	read_req <= '1';
--	
--	if(local_rdata_valid = '1') then
--		teller1_update <= '1';
--		if(local_rdata = teller1) then 
--			pnf_persist <= '0';--RETT
--		else
--			pnf_persist <= '1';--FEIL
--		end if;
--		if(teller1 > x"00000000000000FF") then
--			next_state <= finish_state;
--		else
--			next_state <= les_state;
--		end if;
--	else	
--		pnf_persist <= '0';
--		teller1_update <= '0';
--		next_state <= les_state;
--	end if;
--	
--	when finish_state =>
--		pnf_per_byte <= "00000000";
--		pnf_persist <= '0';
--		test_status <=  "00000000";
--		test_complete <= '1';
--		read_req <= '0';
--		wdata <= (OTHERS => '0');
--		write_req <= '0';
--		reset_address <= '1';
--		teller1_update <= '0';
--		teller1_reset <= '1';
--		teller2_update <= '0';
--		teller2_reset <= '1';
--		next_state <= pause_state;
--		
--	when pause_state =>
--		pnf_per_byte <= "00000000";
--		pnf_persist <= '0';
--		test_status <=  "00000000";
--		test_complete <= '0';
--		read_req <= '0';
--		wdata <= (OTHERS => '0');
--		write_req <= '0';
--		reset_address <= '1';
--		teller2_update <= '0';
--		teller2_reset <= '1';
--		if(teller1 > x"0000000000000010") then
--			next_state <= idle;
--			teller1_update <= '0';
--			teller1_reset <= '1';
--		else
--			next_state <= pause_state;
--			teller1_update <= '1';
--			teller1_reset <= '0';
--		end if;
--		
--end case;
--end process;
--
--process(clk, reset_n, reset_address)
--begin
--if(reset_address = '1') then
--	col_addr <= (OTHERS => '0');
--	row_addr <= (OTHERS => '0');
--	bank_addr <= (OTHERS => '0');
--elsif(reset_n = '0') then
--	col_addr <= (OTHERS => '0');
--	row_addr <= (OTHERS => '0'); 
--	bank_addr <= (OTHERS => '0');
--elsif(rising_edge(clk)) then
--if ((local_ready = '1' AND write_req = '1') OR (local_ready = '1' AND read_req = '1')) then
--        if col_addr>= MAX_COL then 
--          col_addr <= (OTHERS=>'0');
--          if row_addr = MAX_ROW then 
--            row_addr <= (OTHERS=>'0');
--            if bank_addr = MAX_BANK then 
--              bank_addr <= (OTHERS=>'0');
--				  --ferdig <= '1';
--            else
--              bank_addr <= bank_addr + 1; 
--				 -- ferdig <= '0';
--            end if;
--          else
--            row_addr <= row_addr + 1; 
--          end if;
--        else
--          col_addr <= col_addr + 8;--eller 4
--        end if;
--      end if;
--end if;
--end process;
--	 
--end rtl;	

--process(clk, reset_n) 
--begin
--if(reset_n = '0') then
--	teller1 <= (OTHERS => '0');
--	teller2 <= (OTHERS => '0');
--	--teller3 <= (OTHERS => '0');
--	current_state <= idle;
--elsif(rising_edge(clk)) then
--	current_state <= next_state;
--	if(teller1_update = '1') then
--		teller1 <= teller1 + 1;
--	elsif(teller1_reset = '1') then
--		teller1 <= (OTHERS => '0');
--	end if;
--	if(teller2_update = '1') then
--		teller2 <= teller2 + 1;
--	elsif(teller2_reset = '1') then
--		teller2 <= (OTHERS => '0');
--	end if;
----	if(teller3_update = '1') then
----		teller3 <= teller3 + 1;
----	elsif(teller3_reset = '1') then
----		teller3 <= (OTHERS => '0');
----	end if;
--end if;
--end process;
--
--process(current_state, teller1, teller2, local_ready, local_rdata_valid, local_rdata, local_wdata_req)--, start)
--begin
--case current_state is
--
--when idle =>
--	pnf_per_byte <= "00000000";
--	pnf_persist <= '0';
--	test_status <=  "00000000";
--	test_complete <= '0';
--	--burstbegin <= '0';
--	read_req <= '0';
--   wdata <= (OTHERS => '0');
--	write_req <= '0';
--	reset_address <= '1';
--	pnf_persist <= '0';
--	teller1_update <= '0';
--	teller1_reset <= '0';
--	teller2_update <= '0';
--	teller2_reset <= '0';
--	--teller3_update <= '0';
--	--teller3_reset <= '0';
--	--if (start = '0') then
--		next_state <= skriv_state;
--	--else
--		--next_state <= idle;
--	--end if;
--	
--when skriv_state =>
--	test_complete <= '0';
--	pnf_per_byte <= "00000000";
--	pnf_persist <= '0';
--	test_status <=  "00000000";
--	read_req <= '0';
--	reset_address <= '0';
--	teller2_update <= '0';
--	teller2_reset <= '0';
--	--teller1_reset <= '0';
--	--burstbegin <= '0';
--	if(local_ready = '1') then
--		
----			write_req <= '0';
----			--burstbegin <= '0';
----			wdata <= (OTHERS => '0');
----			teller1_update <= '0';
----			teller1_reset <= '1';
----			--teller3_update <= '0';
----			--teller3_reset <= '1';
----		else
----			next_state <= skriv_state;
--			write_req <= '1';
--	else
--		write_req <= '0';
--			--wdata <= teller1;
--			--teller1_update <= '1';
--			--teller1_reset <= '0';
--			--burstbegin <= '1';
----			if(teller3 = "0000") then
----				burstbegin <= '1';
----				teller3_update <= '1';
----				teller3_reset <= '0';
----			else
----				burstbegin <= '0';
----				teller3_reset <= '1';
----				teller3_update <= '0';
----			end if;
--			
--		end if;
----	else
----		write_req <= '0';
----		wdata <= (OTHERS => '0');
----		next_state <= skriv_state;
----		teller1_update <= '0';
----		teller1_reset <= '0';
----		--teller3_update <= '0';
----		--teller3_reset <= '0';
----		--burstbegin <= '0';
----	end if;
--	
--	if(local_wdata_req = '1') then
--		wdata <= teller1;
--		teller1_update <= '1';
--	else	
--		teller1_update <= '0';
--		wdata <= (OTHERS => '0');
--	end if;
--	
--	if(teller1 > "1111111111111111111111111111111111111111111111111111111111110000") then------------------------------------------------
--		next_state <= reset_state;
--		teller1_reset <= '1';
--	else
--		next_state <= skriv_state;
--		teller1_reset <= '0';
--	end if;
--	
--when reset_state =>
--	pnf_per_byte <= "00000000";
--	pnf_persist <= '0';
--	test_status <=  "00000000";
--	test_complete <= '0';
--	--next_state <= les_state;
--	--burstbegin <= '0';
--	read_req <= '0';
--   wdata <= (OTHERS => '0');
--	write_req <= '0';
--	
--	teller2_update <= '0';
--	teller2_reset <= '1';
--	if(teller1 > "1111111111111111111111111111111111111111111111111111111111100100") then
--		reset_address <= '1';
--		teller1_update <= '0';
--		teller1_reset <= '1';
--		next_state <= les_state;
--	else
--		reset_address <= '0';
--		teller1_update <= '1';
--		teller1_reset <= '0';
--		next_state <= reset_state;
--	end if;
--	
--	--teller3_update <= '0';
--	--teller3_reset <= '0';
--	
--when les_state =>
--	test_complete <= '0';
--	--burstbegin <= '0';
--	pnf_per_byte <= "00000000";
--	test_status <=  "00000000";
--	wdata <= (OTHERS => '0');
--	write_req <= '0';
--	reset_address <= '0';
--	teller1_reset <= '0';
--	teller2_reset <= '0';
--	--teller3_update <= '0';
--	--teller3_reset <= '0';
--	if(local_ready = '1') then
--		if(teller2 > "1111111111111111111111111111111111111111111111111111111111110000") then
--			read_req <= '0';
--			teller2_update <= '0';
--			--burstbegin <= '0';
--			--teller3_update <= '0';
--			--teller3_reset <= '0';
--		else
--			read_req <= '1';
--			teller2_update <= '1';
--			--burstbegin <= '1';
----			if(teller3 = "0000") then
----				burstbegin <= '1';
----				teller3_update <= '1';
----				teller3_reset <= '0';
----			else
----				burstbegin <= '0';
----				teller3_reset <= '1';
----				teller3_update <= '0';
----			end if;
--			
--		end if;
--	else
--		read_req <= '0';
--		teller2_update <= '0';
--		--burstbegin <= '0';
--		--teller3_update <= '0';
--		--teller3_reset <= '0';
--	end if;
--		
--		
--	if(local_rdata_valid = '1') then
--		teller1_update <= '1';
--		if(local_rdata = teller1) then 
--			pnf_persist <= '0';
--		else
--			pnf_persist <= '1';--FEIL
--		end if;
--	else	
--		pnf_persist <= '0';
--		teller1_update <= '0';
--	end if;
--	
--	if(teller1 > "1111111111111111111111111111111111111111111111111111111111110000") then------------------------------------------------
--		next_state <= finish_state;
--	else
--		next_state <= les_state;
--	end if;
--	
--	when finish_state =>
--		pnf_per_byte <= "00000000";
--		pnf_persist <= '0';
--		test_status <=  "00000000";
--		test_complete <= '1';
--		--burstbegin <= '0';
--		read_req <= '0';
--		wdata <= (OTHERS => '0');
--		write_req <= '0';
--		reset_address <= '1';
--		teller1_update <= '0';
--		teller1_reset <= '1';
--		teller2_update <= '0';
--		teller2_reset <= '1';
--		--teller3_update <= '0';
--		--teller3_reset <= '1';
--		next_state <= pause_state;
--		
--	when pause_state =>
--		pnf_per_byte <= "00000000";
--		pnf_persist <= '0';
--		test_status <=  "00000000";
--		test_complete <= '0';
--		--burstbegin <= '0';
--		read_req <= '0';
--		wdata <= (OTHERS => '0');
--		write_req <= '0';
--		reset_address <= '1';
--		teller2_update <= '0';
--		teller2_reset <= '1';
--		--teller3_update <= '0';
--		--teller3_reset <= '0';
--		if(teller1 > "1111111111111111111111111111111111111111111111111111111111110000") then
--			next_state <= idle;--reset_state;--idle;
--			teller1_update <= '0';
--			teller1_reset <= '1';
--		else
--			next_state <= pause_state;
--			teller1_update <= '1';
--			teller1_reset <= '0';
--		end if;
--		
--end case;
--end process;
--
--process(clk, reset_n, reset_address)
--begin
--if(reset_address = '1') then
--	col_addr <= (OTHERS => '0');
--	row_addr <= (OTHERS => '0');
--	bank_addr <= (OTHERS => '0');
--elsif(reset_n = '0') then
--	col_addr <= (OTHERS => '0');
--	row_addr <= (OTHERS => '0'); 
--	bank_addr <= (OTHERS => '0');
--elsif(rising_edge(clk)) then
--if ((local_wdata_req = '1') OR (local_ready = '1' AND read_req = '1')) then --local_ready = '1' AND 
--        if col_addr>= MAX_COL then 
--          col_addr <= (OTHERS=>'0');
--          if row_addr = MAX_ROW then 
--            row_addr <= (OTHERS=>'0');
--            if bank_addr = MAX_BANK then 
--              bank_addr <= (OTHERS=>'0');
--				  --ferdig <= '1';
--            else
--              bank_addr <= bank_addr + 1; 
--				 -- ferdig <= '0';
--            end if;
--          else
--            row_addr <= row_addr + 1; 
--          end if;
--        else
--          col_addr <= col_addr + 8;
--        end if;
--      end if;
--end if;
--end process;
--	 
--end rtl;	