LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;

ENTITY data_shift_reg_en IS
generic (length : integer := 2
			);
PORT (
	clk 		: 			IN STD_LOGIC;
	reset    :        IN STD_LOGIC;
	enable   :        IN STD_LOGIC;
	inn 		: 			IN STD_LOGIC_VECTOR(63 DOWNTO 0);
	ut 		: 			OUT STD_LOGIC_VECTOR(63 DOWNTO 0)
);
END data_shift_reg_en;

ARCHITECTURE rtl OF data_shift_reg_en IS

SUBTYPE sr_width IS STD_LOGIC_VECTOR(63 DOWNTO 0);
TYPE sr_length IS ARRAY (length-1 DOWNTO 0) OF sr_width;

SIGNAL sr : sr_length;

BEGIN
	process (clk, reset)
begin
	if(reset = '1') then
		sr(0) <= (OTHERS => '0');
		for i in 1 to length-1 loop
			sr(i) <= (OTHERS => '0');
		end loop;

	elsif (rising_edge(clk)) then

		if (enable = '1') then

			-- Shift data by one stage; data from last stage is lost
			sr((length-1) downto 1) <= sr((length-2) downto 0);

			-- Load new data into the first stage
			sr(0) <= inn;

		end if;
	end if;
end process;

-- Capture the data from the last stage, before it is lost
ut <= sr(length-1);

END rtl;