library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity dot_product is
	port
   (
      data_a, data_b		 : in std_logic_vector(63 downto 0); 
		reset, clk, start, mode  : in std_logic;
		size : in std_logic_vector(12 downto 0);--12 downto 0 -------------------------------------------------------------
		done_rr : out std_logic;
		done_dq : out std_logic;
		result : out std_logic_vector(63 downto 0)
   );
end entity dot_product;

architecture rtl of dot_product is

component control_dot_product
	port(
		clk : in std_logic;
		reset : in std_logic;
		size : in std_logic_vector(12 downto 0);--12 downto 0--------------------------------------------------------------------------------------
		start : in std_logic;
		mode : in std_logic;
		calculate : out std_logic;
		reset_data_reg : out std_logic;
		d_sel1 : out std_logic;
		d_sel2 : out std_logic_vector(2 downto 0);
		d_sel3 : out std_logic;
		done_rr : out std_logic;
		done_dq : out std_logic
	);
end component;
  
  component data_mux2
  port
   (
      a, b : in std_logic_vector(63 downto 0); 
      sel : in std_logic;
		result : out std_logic_vector(63 downto 0)
   );
	end component;
	
	component data_mux8
   port
   (
      a, b, c, d, e, f, g, h : in std_logic_vector(63 downto 0); 
      sel : in std_logic_vector(2 downto 0);
		result : out std_logic_vector(63 downto 0)
   );
	end component;
	
  component multiplier_ny
    PORT
	(
		aclr     : IN STD_LOGIC;
		clock		: IN STD_LOGIC ;
		dataa		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
		datab		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
		result		: OUT STD_LOGIC_VECTOR (63 DOWNTO 0)
	);
  end component;
  
  component add_ny
  PORT
	(
		aclr     : IN STD_LOGIC;
		clock		: IN STD_LOGIC ;
		dataa		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
		datab		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
		result		: OUT STD_LOGIC_VECTOR (63 DOWNTO 0)
	);
	end component;
	
	component data_shift_reg_tap
		generic (length : integer := 4
		);
		PORT (
			clk 		: 			IN STD_LOGIC;
			global_reset : IN STD_LOGIC;
			reset    :        IN STD_LOGIC;
			inn 		: 			IN STD_LOGIC_VECTOR(63 DOWNTO 0);
			tap1     :        OUT STD_LOGIC_VECTOR(63 downto 0);
			tap2     :        OUT STD_LOGIC_VECTOR(63 downto 0);
			tap3     :        OUT STD_LOGIC_VECTOR(63 downto 0);
			ut 		: 			OUT STD_LOGIC_VECTOR(63 DOWNTO 0)
		);
	end component;
	
	component control_shift_reg
		generic (length : integer := 12
			);
		PORT (
			clk 		: 			IN STD_LOGIC;
			reset    :        IN STD_LOGIC;
			inn 		: 			IN STD_LOGIC;
			ut 		: 			OUT STD_LOGIC
		);
   end component;
  
  
  --signals
  signal d_wire1, d_wire2, d_wire3, d_wire4, d_wire5, d_wire6, d_wire7, d_wire8, d_wire9, d_wire10, d_wire11, d_wire12 : std_logic_vector(63 downto 0);
  
  signal c_wire1, c_wire2, c_wire3 : std_logic;
  signal d_sel1, d_sel3 : std_logic; 
  signal d_sel2 : std_logic_vector(2 downto 0);
  
  constant length1 : integer := 8;
  constant length2 : integer := 11;
  
  begin
  
   control : control_dot_product PORT MAP (
		clk => clk,
		reset => reset,
		size => size,
		start => start,
		mode => mode,
		calculate => c_wire1,
		reset_data_reg => c_wire3,
		--reset_adder => c_wire4,
		d_sel1 => d_sel1,
		d_sel2 => d_sel2,
		d_sel3 => d_sel3,
		done_rr => done_rr,
		done_dq => done_dq
	);
  
   multiplier_dot_product : multiplier_ny PORT MAP (
		aclr => reset,
		clock => clk,
		dataa	 => data_a, --d_wire9,
		datab	 => data_b, --d_wire10,
		result	 => d_wire1
	);
	
	add_dot_product : add_ny PORT MAP (
		aclr => reset,--c_wire4
		clock => clk,
		dataa	 => d_wire2,
		datab	 => d_wire3,
		result	 => d_wire11 --d_wire4
	);
	
	d_mux1 : data_mux2 port map (
		a => d_wire1,
		b => d_wire4,
		sel => c_wire2,
		result => d_wire2
	);
	
	d_mux2 : data_mux2 port map (
		a => d_wire4,
		b => (OTHERS => '0'),
		sel => d_sel1,
		result => d_wire8
	);
	
	d_mux3 : data_mux8 port map (
		a => d_wire4,
		b => d_wire5,
		c => d_wire6,
		d => d_wire7,
		e => d_wire12,
		f => (OTHERS => '0'),
		g => (OTHERS => '0'),
		h => (OTHERS => '0'),
		sel => d_sel2,
		result => d_wire3
	);
--	
--	innmux1 : data_mux2 port map (
--		a => data_a,
--		b => (OTHERS => '0'),
--		sel => c_wire1,
--		result => d_wire9
--		);
--	
--	innmux2 : data_mux2 port map (
--		a => data_b,
--		b => (OTHERS => '0'),
--		sel => c_wire1,
--		result => d_wire10
--		);
		
	utmux1 : data_mux2 port map (
		a => d_wire11,
		b => (OTHERS => '0'),
		sel => d_sel3,
		result => d_wire4
  	);
		
	data_shift_reg_tap1 : data_shift_reg_tap
	Generic map ( length =>  length1)
	port map (
		clk => clk,
		global_reset => reset,
		reset => c_wire3,
		inn => d_wire8,
		tap1 => d_wire5,
		tap2 => d_wire6,
		tap3 => d_wire12,
		ut => d_wire7
	);
	
	control_shift_reg1 : control_shift_reg
  GENERIC MAP (length => length2)
  PORT MAP (
		clk => clk,
		reset => reset,
		inn => c_wire1,
		ut	 => c_wire2
	);
	
	result <= d_wire4;
	
end rtl;
--library IEEE;
--use IEEE.STD_LOGIC_1164.all;
--
--entity dot_product is
--	port
--   (
--      data_a, data_b		 : in std_logic_vector(63 downto 0); 
--		reset, clk, start, mode  : in std_logic;
--		size : in std_logic_vector(3 downto 0);--12 downto 0 -------------------------------------------------------------
--		done_rr : out std_logic;
--		done_dq : out std_logic;
--		result : out std_logic_vector(63 downto 0)
--   );
--end entity dot_product;
--
--architecture rtl of dot_product is
--
--component control_dot_product
--	port(
--		clk : in std_logic;
--		reset : in std_logic;
--		size : in std_logic_vector(3 downto 0);--12 downto 0--------------------------------------------------------------------------------------
--		start : in std_logic;
--		mode : in std_logic;
--		calculate : out std_logic;
--		d_sel1 : out std_logic;
--		d_sel2 : out std_logic;
--		d_sel3 : out std_logic;
--		d_sel4 : out std_logic_vector(1 downto 0);
--		d_sel5 : out std_logic;
--		done_rr : out std_logic;
--		done_dq : out std_logic
--	);
--end component;
--  
--  component data_mux2
--  port
--   (
--      a, b : in std_logic_vector(63 downto 0); 
--      sel : in std_logic;
--		result : out std_logic_vector(63 downto 0)
--   );
--	end component;
--	
--	component data_mux4
--  port
--   (
--      a, b, c, d : in std_logic_vector(63 downto 0); 
--      sel : in std_logic_vector(1 downto 0);
--		result : out std_logic_vector(63 downto 0)
--   );
--	end component;
--	
--	component control_mux2
--	port
--   (
--      a, b : in std_logic; 
--      sel : in std_logic;
--		result : out std_logic
--   );
--	end component;
--	
--	component reg
--	PORT (
--		clk 		: 			IN STD_LOGIC;
--		reset    :        IN STD_LOGIC;
--		inn 		: 			IN STD_LOGIC_VECTOR(63 DOWNTO 0);
--		ut 		: 			OUT STD_LOGIC_VECTOR(63 DOWNTO 0)
--   );
--   end component;
--	
--  component multiplier
--    PORT
--	(
--		aclr     : IN STD_LOGIC;
--		clock		: IN STD_LOGIC ;
--		dataa		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
--		datab		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
--		result		: OUT STD_LOGIC_VECTOR (63 DOWNTO 0)
--	);
--  end component;
--  
--  component add
--  PORT
--	(
--		aclr     : IN STD_LOGIC;
--		clock		: IN STD_LOGIC ;
--		dataa		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
--		datab		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
--		result		: OUT STD_LOGIC_VECTOR (63 DOWNTO 0)
--	);
--	end component;
--	
--	component data_shift_reg
--		generic (length : integer := 3
--			);
--		PORT (
--			clk 		: 			IN STD_LOGIC;
--			inn 		: 			IN STD_LOGIC_VECTOR (63 DOWNTO 0);
--			ut 		: 			OUT STD_LOGIC_VECTOR (63 DOWNTO 0)
--		);
--	end component;
--	
--	component shift_reg2
--		generic (length : integer
--			);
--		PORT (
--			clk 		: 			IN STD_LOGIC;
--			inn 		: 			IN STD_LOGIC;
--			tap1		:        OUT STD_LOGIC;
--			ut 		: 			OUT STD_LOGIC
--		);
--   end component;
--  
--  
--  --signals
--  signal d_wire1, d_wire2, d_wire3, d_wire4, d_wire5, d_wire6, d_wire7, d_wire8, d_wire9, d_wire10, d_wire11, d_wire12, d_wire13 : std_logic_vector(63 downto 0);
--  
--  signal c_wire1, c_wire2, c_wire3, c_wire4, c_wire5 : std_logic;
--  signal d_sel1, d_sel2, d_sel3, d_sel5 : std_logic; 
--  signal d_sel4 : std_logic_vector(1 downto 0);
--  
--  constant length1 : integer := 2;
--  constant length2 : integer := 4;
--  constant length3 : integer := 12;
--  
--  begin
--  
--   control : control_dot_product PORT MAP (
--		clk => clk,
--		reset => reset,
--		size => size,
--		start => start,
--		mode => mode,
--		calculate => c_wire3,---------------------------
--		d_sel1 => d_sel1,
--		d_sel2 => d_sel2,
--		d_sel3 => d_sel3,
--		d_sel4 => d_sel4,
--		d_sel5 => d_sel5,
--		done_rr => done_rr,--c_wire5
--		done_dq => done_dq
--	);
--  
--   multiplier_dot_product : multiplier PORT MAP (
--		aclr => reset,
--		clock => clk,
--		dataa	 => data_a,
--		datab	 => data_b,
--		result	 => d_wire1
--	);
--	
--	add_dot_product : add PORT MAP (
--		aclr => reset,
--		clock => clk,
--		dataa	 => d_wire2,
--		datab	 => d_wire3,
--		result	 => d_wire13
--	);
--	
--	d_mux1 : data_mux2 port map (
--		a => d_wire4,
--		b => (OTHERS => '0'),
--		sel => c_wire1,
--		result => d_wire5
--	);
--	
--	d_mux2 : data_mux2 port map (
--		a => d_wire4,
--		b => (OTHERS => '0'),
--		sel => d_sel1,
--		result => d_wire6
--	);
--	
--	d_mux3 : data_mux2 port map (
--		a => d_wire4,
--		b => (OTHERS => '0'),
--		sel => d_sel2,
--		result => d_wire8
--	);
--	
--	d_mux4 : data_mux2 port map (
--		a => d_wire4,
--		b => (OTHERS => '0'),
--		sel => d_sel3,
--		result => d_wire10
--	);
--	
--	d_mux5 : data_mux2 port map (
--		a => d_wire1,
--		b => (OTHERS => '0'),
--		sel => c_wire2,
--		result => d_wire12
--	);
--	
--	d_mux6 : data_mux4 port map (
--		a => d_wire5,
--		b => d_wire7,
--		c => d_wire9,
--		d => d_wire11,
--		sel => d_sel4,
--		result => d_wire3
--	);
--	
--	d_mux7 : data_mux2 port map (
--		a => d_wire12,
--		b => d_wire4,
--		sel => c_wire2,
--		result => d_wire2
--	);
--	
--	d_mux8 : data_mux2 port map (
--		a => d_wire13,
--		b => (OTHERS => '0'),
--		sel => d_sel5,
--		result => d_wire4
--	);
--	
--	reg1 : reg port map (
--		clk => clk,
--		reset => reset,
--		inn => d_wire6,
--		ut => d_wire7
--	);
--	
--	data_shift_reg1 : data_shift_reg
--  GENERIC MAP (length => length1)
--  PORT MAP (
--		clk => clk,
--		inn => d_wire8,
--		ut	 => d_wire9
--	);
--	
--	data_shift_reg2 : data_shift_reg
--	Generic map ( length =>  length2)
--	port map (
--		clk => clk,
--		inn => d_wire10,
--		ut => d_wire11
--	);
--	
--	shift_reg_control : shift_reg2
--  GENERIC MAP (length => length3)
--  PORT MAP (
--		clk => clk,
--		inn => c_wire3,
--		tap1 => c_wire2,
--		ut	 => c_wire1
--	);
--	
--	result <= d_wire4;
--	
--end rtl;