library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.std_logic_unsigned.all;
 
entity fill_fifo_control is
   port
   (
		clk, reset, start, fifo1_full, fifo2_full : in std_logic;
      size : in std_logic_vector(3 downto 0); 
		wrusedw1, wrusedw2 : in std_logic_vector(7 downto 0);
		wr_fifo1, wr_fifo2, rd_matrix : out std_logic
		--done : out std_logic
      
	);
end entity fill_fifo_control;
 
architecture rtl of fill_fifo_control is
type state_type is (idle, read_matrix_state, state_one, state_two);--, finished_state);
signal current_state, next_state : state_type;

signal counter : std_logic_vector(3 downto 0);
signal count_number_of_writes : std_logic_vector(7 downto 0);
signal update_counter, reset_counter, update_count_number_of_writes, reset_count_number_of_writes : std_logic;

begin

process(clk, reset)
	begin
		if(reset = '1') then
			counter <= (OTHERS => '0');
			current_state <= idle;
			count_number_of_writes <= (OTHERS => '0');
		elsif(rising_edge(clk)) then
			current_state <= next_state;
			if(update_counter = '1') then
				counter <= counter + 1;
			elsif(reset_counter = '1') then
				counter <= (OTHERS => '0');
			end if;
			if(update_count_number_of_writes = '1') then
				count_number_of_writes <= count_number_of_writes + 1;
			elsif(reset_count_number_of_writes = '1') then
				count_number_of_writes <= (OTHERS => '0');
			end if;	
		end if;
	end process;
			
	
	process(current_state, size, start, counter, wrusedw1, wrusedw2, count_number_of_writes)
	begin
	case current_state is
		when idle =>
			update_counter <= '0';
			reset_counter <= '1';
			reset_count_number_of_writes <= '1';
			update_count_number_of_writes <= '0';
			--done <= '0';
			wr_fifo1 <= '0';
			wr_fifo2 <= '0';
			rd_matrix <= '0';
			--id <= '0';
			if(start = '1') then
				next_state <= read_matrix_state;
			else
				next_state <= idle;
			end if;
			
		when read_matrix_state =>
			reset_count_number_of_writes <= '1';
			update_count_number_of_writes <= '0';
			reset_counter <= '0';
			update_counter <= '0';
			--done <= '0';
			wr_fifo1 <= '0';
			wr_fifo2 <= '0';
			rd_matrix <= '0';
			if(wrusedw1 < "00100000" and wrusedw2 < "00100000") then
				if(counter = "0000") then
					next_state <= state_one;
				elsif(counter = "0001") then
					next_state <= state_two;
				else
					next_state <= state_one;
				end if;
			else
				next_state <= read_matrix_state;
			end if;
			
		when state_one => --write to fifo1
			--done <= '0';
			wr_fifo2 <= '0';
			if(count_number_of_writes < "00001000") then --have to be a multiple of number of fifos, for example if it is two fifoes it have to be an even number
				rd_matrix <= '1';
				wr_fifo1 <= '1';
				update_count_number_of_writes <= '1';
				reset_count_number_of_writes <= '0';
				if(counter = size - 1) then
					next_state <= state_one;
					update_counter <= '0';
					reset_counter <= '1';
				else
					next_state <= state_two;
					reset_counter <= '0';
					update_counter <= '1';
				end if;
			else
				rd_matrix <= '0';
				wr_fifo1 <= '0';
				next_state <= read_matrix_state;
				update_count_number_of_writes <= '0';
				reset_count_number_of_writes <= '1';
				reset_counter <= '0';
				update_counter <= '0';
			end if;
			
		when state_two => --write to fifo2
			--done <= '0';
			wr_fifo1 <= '0';
			if(count_number_of_writes < "00001000") then
				rd_matrix <= '1';
				wr_fifo2 <= '1';
				update_count_number_of_writes <= '1';
				reset_count_number_of_writes <= '0';
				if(counter = size - 1) then
					next_state <= state_one;
					update_counter <= '0';
					reset_counter <= '1';
				else
					next_state <= state_one;
					reset_counter <= '0';
					update_counter <= '1';
				end if;
			else
				rd_matrix <= '0';
				wr_fifo2 <= '0';
				next_state <= read_matrix_state;
				update_count_number_of_writes <= '0';
				reset_count_number_of_writes <= '1';
				reset_counter <= '0';
				update_counter <= '0';
			end if;
		
--		when finished_state =>
--			done <= '1';
--			wr_memory1 <= '0';
--			wr_memory2 <= '0';
--			reset_counter <= '1';
--			update_counter <= '0';
--			next_state <= idle;
		end case;
	end process;
end architecture rtl;