library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.std_logic_unsigned.all;
 
entity fill_memory_control is
   port
   (
		clk, reset, start : in std_logic;
      size : in std_logic_vector(12 downto 0); 
		wr_memory1, wr_memory2 : out std_logic;
		done : out std_logic
      
	);
end entity fill_memory_control;
 
architecture rtl of fill_memory_control is
type state_type is (idle, wait_state1, state_one, state_two, finished_state);
signal current_state, next_state : state_type;

signal counter : std_logic_vector(12 downto 0);
signal update_counter, reset_counter : std_logic;

begin

process(clk, reset)
	begin
		if(reset = '1') then
			counter <= (OTHERS => '0');
			current_state <= idle;
		elsif(rising_edge(clk)) then
			current_state <= next_state;
			if(update_counter = '1') then
				counter <= counter + 1;
			elsif(reset_counter = '1') then
				counter <= (OTHERS => '0');
			end if;
		end if;
	end process;
			
	
	process(current_state, size, start, counter)
	begin
	case current_state is
		when idle =>
			update_counter <= '0';
			reset_counter <= '1';
			done <= '0';
			wr_memory1 <= '0';
			wr_memory2 <= '0';
			if(start = '1') then
				next_state <= wait_state1;
			else
				next_state <= idle;
			end if;
			
		when wait_state1 =>
			update_counter <= '0';
			reset_counter <= '1';
			done <= '0';
			wr_memory1 <= '0';
			wr_memory2 <= '0';
			next_state <= state_one;
--			
--		when wait_state2 =>
--			update_counter <= '0';
--			reset_counter <= '1';
--			done <= '0';
--			wr_memory1 <= '0';
--			wr_memory2 <= '0';
--			next_state <= state_one;
			
		when state_one =>
			reset_counter <= '0';
			done <= '0';
			wr_memory2 <= '0';
			if(counter = size-1) then--counter = size
				next_state <= finished_state;
				update_counter <= '0';
				wr_memory1 <= '1';---0
			else
				next_state <= state_two;
				update_counter <= '1';
				wr_memory1 <= '1';
			end if;
			
		when state_two =>
			reset_counter <= '0';
			done <= '0';
			wr_memory1 <= '0';
			
			if(counter = size-1) then--counter = size
				next_state <= finished_state;
				update_counter <= '0';
				wr_memory2 <= '1';---0
			else
				next_state <= state_one;
				update_counter <= '1';
				wr_memory2 <= '1';
			end if;
		
		when finished_state =>
			done <= '1';
			wr_memory1 <= '0';
			wr_memory2 <= '0';
			reset_counter <= '1';
			update_counter <= '0';
			next_state <= idle;
		end case;
	end process;
end architecture rtl;