library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity matrix_vector_multiplicator is
	port
   (
      inndata1, inndata2, inndata3, inndata4		 : in std_logic_vector(63 downto 0); 
		reset, clk, start, fifo1_empty, fifo2_empty  : in std_logic;
		size1, size2 : in std_logic_vector(11 downto 0);
		size : in std_logic_vector(12 downto 0);--12 downto 0-------------------------------------------------------------------------------------------------
		done, re_fifo1, re_fifo2, re_memory1, re_memory2, row_done : out std_logic;
		result : out std_logic_vector(63 downto 0)
   );
end entity matrix_vector_multiplicator;

architecture rtl of matrix_vector_multiplicator is

component matrix_vector_multiplicator_control
	PORT
	(
		clk, reset, start, empty_fifo1, empty_fifo2 : in std_logic;
		size       : in std_logic_vector(12 downto 0);--12 downto 0-------------------------------------------------------------------------------------
		sel1, sel2, sel3, sel4, re_fifo1, re_fifo2, re_memory1, re_memory2 : out std_logic;
		size1, size2 : in std_logic_vector(11 downto 0);
		done : out std_logic
	);
end component;
		
component multiplier_ny
	PORT
	(
		aclr		: IN STD_LOGIC ;
		clock		: IN STD_LOGIC ;
		dataa		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
		datab		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
		result		: OUT STD_LOGIC_VECTOR (63 DOWNTO 0)
	);
end component;

component add_ny
	PORT
	(
		aclr		: IN STD_LOGIC ;
		clock		: IN STD_LOGIC ;
		dataa		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
		datab		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
		result		: OUT STD_LOGIC_VECTOR (63 DOWNTO 0)
	);
end component;

component data_mux2
	PORT
	(
      a, b : in std_logic_vector(63 downto 0); 
      sel : in std_logic;
		result : out std_logic_vector(63 downto 0)
   );
end component;

component data_shift_reg_ny
generic (length : integer := 2
			);
PORT (
	clk 		: 			IN STD_LOGIC;
	reset    :        IN STD_LOGIC;
	inn 		: 			IN STD_LOGIC_VECTOR(63 DOWNTO 0);
	ut 		: 			OUT STD_LOGIC_VECTOR(63 DOWNTO 0)
);
end component;

component control_shift_reg
generic (length : integer := 2
			);
PORT (
	clk 		: 			IN STD_LOGIC;
	reset    :        IN STD_LOGIC;
	inn 		: 			IN STD_LOGIC;
	ut 		: 			OUT STD_LOGIC
);
end component;

component reg
PORT (
	clk 		: 			IN STD_LOGIC;
	reset    :        IN STD_LOGIC;
	inn 		: 			IN STD_LOGIC_VECTOR(63 DOWNTO 0);
	ut 		: 			OUT STD_LOGIC_VECTOR(63 DOWNTO 0)
);
end component;

component flip_flop
PORT (
	clk 		: 			IN STD_LOGIC;
	reset    :        IN STD_LOGIC;
	inn 		: 			IN STD_LOGIC;
	ut 		: 			OUT STD_LOGIC
);
end component;

signal d_wire1, d_wire2, d_wire3, d_wire4, d_wire5, d_wire6, d_wire7, d_wire8, d_wire9, d_wire10, d_wire11, d_wire12, d_wire13, d_wire14, d_wire15, d_wire16, d_wire17, d_wire18 : std_logic_vector(63 downto 0);
signal c_wire1, c_wire2, c_wire3, c_wire4, c_wire5, c_wire6, c_wire7, c_wire8, c_wire9 : std_logic;
--Add_latency=14
--Mult_latency=11

constant length1 : integer := 2;
constant length2 : integer := 4;
constant length3 : integer := 26;--13 
constant length4 : integer := 81;--33
constant length5 : integer := 81;--33
constant length6 : integer := 8;

begin

multiplier1 : multiplier_ny PORT MAP (
	   aclr => reset,
		clock => clk,
		dataa	 => d_wire6,
		datab	 => d_wire7,
		result	 => d_wire1
	);
	
multiplier2 : multiplier_ny PORT MAP (
	   aclr => reset,
		clock => clk,
		dataa	 => d_wire8,
		datab	 => d_wire9,
		result	 => d_wire2
	);
	
adder1 : add_ny PORT MAP (
	   aclr		=> reset,
		clock		=> clk,
		dataa		=> d_wire1,
		datab		=> d_wire2,
		result	=> d_wire3 --d_wire3
  );
  
accumulator : add_ny PORT MAP (
		aclr => reset,
		clock => clk,
		dataa => d_wire3,
		datab => d_wire4,
		result => d_wire5 --d_wire5
	);
	
d_mux21 : data_mux2 PORT MAP (
		a => (OTHERS => '0'),
		b => d_wire5,
		sel => c_wire3,
		result => d_wire4
	);
	
d_mux22 : data_mux2 PORT MAP (
		a => d_wire5,
		b => (OTHERS => '0'),
		sel => c_wire3,
		result => d_wire10
	);

innput1_mux : data_mux2 PORT MAP (
		a => inndata1,
		b => (OTHERS => '0'),
		sel => c_wire1,
		result => d_wire6
	);
	
innput1_delay : flip_flop PORT MAP (
	clk => clk,
	reset => reset,
	inn => c_wire7,
	ut => c_wire1
	);
	
innput2_mux : data_mux2 PORT MAP (
		a => inndata2,
		b => (OTHERS => '0'),
		sel => c_wire1,
		result => d_wire7
	);
	
innput2_delay : flip_flop PORT MAP (
	clk => clk,
	reset => reset,
	inn => c_wire8,
	ut => c_wire2
	);
	
innput3_mux : data_mux2 PORT MAP (
		a => inndata3,
		b => (OTHERS => '0'),
		sel => c_wire2,
		result => d_wire8
	);
	
innput4_mux : data_mux2 PORT MAP (
		a => inndata4,
		b => (OTHERS => '0'),
		sel => c_wire2,
		result => d_wire9
	);
	
--Accumulate the seven values

adder2 : add_ny PORT MAP (
	   aclr => reset,
		clock => clk,
		dataa => d_wire10,
		datab => d_wire11,
		result => d_wire12
	);

reg1 : reg PORT MAP (
		clk => clk,
		reset => reset,
		inn => d_wire10,
		ut => d_wire11
	);
	
adder3 : add_ny PORT MAP (
	   aclr => reset,
		clock => clk,
		dataa => d_wire12,
		datab => d_wire13,
		result => d_wire14
	);
	
shift_reg1 : data_shift_reg_ny 
	GENERIC MAP ( length => length1)
	PORT MAP (
		clk => clk,
		reset => reset,
		inn => d_wire12,
		ut => d_wire13
	);

adder4 : add_ny PORT MAP (
	   aclr => reset,
		clock => clk,
		dataa => d_wire14,
		datab => d_wire15,
		result => d_wire16
	);
	
shift_reg2 : data_shift_reg_ny 
	GENERIC MAP ( length => length2)
	PORT MAP (
		clk => clk,
		reset => reset,
		inn => d_wire14,
		ut => d_wire15
	);
	
adder5 : add_ny PORT MAP (
	   aclr => reset,
		clock => clk,
		dataa => d_wire16,
		datab => d_wire17,
		result => d_wire18
	);
	
shift_reg3 : data_shift_reg_ny 
	GENERIC MAP ( length => length6)
	PORT MAP (
		clk => clk,
		reset => reset,
		inn => d_wire16,
		ut => d_wire17
	);
	
output_mux : data_mux2 PORT MAP (
		a => d_wire18,
		b => (OTHERS => '0'),
		sel => c_wire4,
		result => result
	);
	
control_shift_reg1 : control_shift_reg
	GENERIC MAP ( length => length3 )
	PORT MAP (
		clk => clk,
		reset => reset,
		inn => c_wire9,
		ut => c_wire3
	);
	
control_shift_reg2 : control_shift_reg
	GENERIC MAP ( length => length4 )
	PORT MAP (
		clk => clk,
		reset => reset,
		inn => c_wire5,
		ut => c_wire4
	);
	
control_shift_reg3 : control_shift_reg
	GENERIC MAP ( length => length5 )
	PORT MAP (
		clk => clk,
		reset => reset,
		inn => c_wire6,
		ut => done
	);
	
matrix_vector_multiplicator_control1 :  matrix_vector_multiplicator_control PORT MAP (
	clk => clk,
	reset => reset,
	start => start,
	empty_fifo1 => fifo1_empty,
	empty_fifo2 => fifo2_empty,
	size => size,
	size1 => size1,
	size2 => size2,
	sel1 => c_wire9,
	sel2 => c_wire7,
	sel3 => c_wire8,
	sel4 => c_wire5,
	re_fifo1 => re_fifo1,
	re_fifo2 => re_fifo2,
	re_memory1 => re_memory1,
	re_memory2 => re_memory2,
	done => c_wire6
);

row_done <= c_wire4;
	
end rtl;	
--library IEEE;
--use IEEE.STD_LOGIC_1164.all;
--
--entity matrix_vector_multiplicator is
--	port
--   (
--      inndata1, inndata2, inndata3, inndata4		 : in std_logic_vector(63 downto 0); 
--		reset, clk, start, fifo1_empty, fifo2_empty  : in std_logic;
--		size1, size2 : in std_logic_vector(3 downto 0);
--		size : in std_logic_vector(3 downto 0);--12 downto 0-------------------------------------------------------------------------------------------------
--		done, re_fifo1, re_fifo2, re_memory1, re_memory2, row_done : out std_logic;
--		result : out std_logic_vector(63 downto 0)
--   );
--end entity matrix_vector_multiplicator;
--
--architecture rtl of matrix_vector_multiplicator is
--
--component matrix_vector_multiplicator_control
--	PORT
--	(
--		clk, reset, start, empty_fifo1, empty_fifo2 : in std_logic;
--		size       : in std_logic_vector(3 downto 0);--12 downto 0-------------------------------------------------------------------------------------
--		sel1, sel2, sel3, sel4, re_fifo1, re_fifo2, re_memory1, re_memory2 : out std_logic;
--		size1, size2 : in std_logic_vector(3 downto 0);
--		done : out std_logic
--	);
--end component;
--		
--component multiplier
--	PORT
--	(
--		aclr		: IN STD_LOGIC ;
--		clock		: IN STD_LOGIC ;
--		dataa		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
--		datab		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
--		result		: OUT STD_LOGIC_VECTOR (63 DOWNTO 0)
--	);
--end component;
--
--component add
--	PORT
--	(
--		aclr		: IN STD_LOGIC ;
--		clock		: IN STD_LOGIC ;
--		dataa		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
--		datab		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
--		result		: OUT STD_LOGIC_VECTOR (63 DOWNTO 0)
--	);
--end component;
--
--component data_mux2
--	PORT
--	(
--      a, b : in std_logic_vector(63 downto 0); 
--      sel : in std_logic;
--		result : out std_logic_vector(63 downto 0)
--   );
--end component;
--
--component data_shift_reg_ny
--generic (length : integer := 2
--			);
--PORT (
--	clk 		: 			IN STD_LOGIC;
--	reset    :        IN STD_LOGIC;
--	inn 		: 			IN STD_LOGIC_VECTOR(63 DOWNTO 0);
--	ut 		: 			OUT STD_LOGIC_VECTOR(63 DOWNTO 0)
--);
--end component;
--
--component control_shift_reg
--generic (length : integer := 2
--			);
--PORT (
--	clk 		: 			IN STD_LOGIC;
--	reset    :        IN STD_LOGIC;
--	inn 		: 			IN STD_LOGIC;
--	ut 		: 			OUT STD_LOGIC
--);
--end component;
--
--component reg
--PORT (
--	clk 		: 			IN STD_LOGIC;
--	reset    :        IN STD_LOGIC;
--	inn 		: 			IN STD_LOGIC_VECTOR(63 DOWNTO 0);
--	ut 		: 			OUT STD_LOGIC_VECTOR(63 DOWNTO 0)
--);
--end component;
--
--component flip_flop
--PORT (
--	clk 		: 			IN STD_LOGIC;
--	reset    :        IN STD_LOGIC;
--	inn 		: 			IN STD_LOGIC;
--	ut 		: 			OUT STD_LOGIC
--);
--end component;
--
--signal d_wire1, d_wire2, d_wire3, d_wire4, d_wire5, d_wire6, d_wire7, d_wire8, d_wire9, d_wire10, d_wire11, d_wire12, d_wire13, d_wire14, d_wire15, d_wire16, d_wire17, d_wire18 : std_logic_vector(63 downto 0);
--signal c_wire1, c_wire2, c_wire3, c_wire4, c_wire5, c_wire6, c_wire7, c_wire8, c_wire9 : std_logic;
--
--constant length1 : integer := 2;
--constant length2 : integer := 4;
--constant length3 : integer := 12;--15
--constant length4 : integer := 33;
--constant length5 : integer := 33;
--
--begin
--
--multiplier1 : multiplier PORT MAP (
--	   aclr => reset,
--		clock => clk,
--		dataa	 => d_wire6,
--		datab	 => d_wire7,
--		result	 => d_wire1
--	);
--	
--multiplier2 : multiplier PORT MAP (
--	   aclr => reset,
--		clock => clk,
--		dataa	 => d_wire8,
--		datab	 => d_wire9,
--		result	 => d_wire2
--	);
--	
--adder1 : add PORT MAP (
--	   aclr		=> reset,
--		clock		=> clk,
--		dataa		=> d_wire1,
--		datab		=> d_wire2,
--		result	=> d_wire3 --d_wire3
--  );
--  
--accumulator : add PORT MAP (
--		aclr => reset,
--		clock => clk,
--		dataa => d_wire3,
--		datab => d_wire4,
--		result => d_wire5 --d_wire5
--	);
--	
--d_mux21 : data_mux2 PORT MAP (
--		a => (OTHERS => '0'),
--		b => d_wire5,
--		sel => c_wire3,
--		result => d_wire4
--	);
--	
--d_mux22 : data_mux2 PORT MAP (
--		a => d_wire5,
--		b => (OTHERS => '0'),
--		sel => c_wire3,
--		result => d_wire10
--	);
--
--innput1_mux : data_mux2 PORT MAP (
--		a => inndata1,
--		b => (OTHERS => '0'),
--		sel => c_wire1,
--		result => d_wire6
--	);
--	
--innput1_mux_sel_delay : flip_flop PORT MAP (
--	clk => clk,
--	reset => reset,
--	inn => c_wire7,
--	ut => c_wire1
--	);
--	
--innput2_mux : data_mux2 PORT MAP (
--		a => inndata2,
--		b => (OTHERS => '0'),
--		sel => c_wire1,
--		result => d_wire7
--	);
--	
--innput2_mux_sel_delay : flip_flop PORT MAP (
--	clk => clk,
--	reset => reset,
--	inn => c_wire8,
--	ut => c_wire2
--	);
--	
--innput3_mux : data_mux2 PORT MAP (
--		a => inndata3,
--		b => (OTHERS => '0'),
--		sel => c_wire2,
--		result => d_wire8
--	);
--	
--innput4_mux : data_mux2 PORT MAP (
--		a => inndata4,
--		b => (OTHERS => '0'),
--		sel => c_wire2,
--		result => d_wire9
--	);
--	
----Accumulate the seven values
--
--adder2 : add PORT MAP (
--	   aclr => reset,
--		clock => clk,
--		dataa => d_wire10,
--		datab => d_wire11,
--		result => d_wire12
--	);
--
--reg1 : reg PORT MAP (
--		clk => clk,
--		reset => reset,
--		inn => d_wire10,
--		ut => d_wire11
--	);
--	
--adder3 : add PORT MAP (
--	   aclr => reset,
--		clock => clk,
--		dataa => d_wire12,
--		datab => d_wire13,
--		result => d_wire14
--	);
--	
--shift_reg1 : data_shift_reg_ny
--	GENERIC MAP ( length => length1)
--	PORT MAP (
--		clk => clk,
--		reset => reset,
--		inn => d_wire12,
--		ut => d_wire13
--	);
--
--adder4 : add PORT MAP (
--	   aclr => reset,
--		clock => clk,
--		dataa => d_wire14,
--		datab => d_wire15,
--		result => d_wire16
--	);
--	
--shift_reg2 : data_shift_reg_ny 
--	GENERIC MAP ( length => length2)
--	PORT MAP (
--		clk => clk,
--		reset => reset,
--		inn => d_wire14,
--		ut => d_wire15
--	);
--	
--output_mux : data_mux2 PORT MAP (
--		a => d_wire16,
--		b => (OTHERS => '0'),
--		sel => c_wire4,
--		result => result
--	);
--	
--control_shift_reg1 : control_shift_reg
--	GENERIC MAP ( length => length3 )
--	PORT MAP (
--		clk => clk,
--		reset => reset,
--		inn => c_wire9,
--		ut => c_wire3
--	);
--	
--control_shift_reg2 : control_shift_reg
--	GENERIC MAP ( length => length4 )
--	PORT MAP (
--		clk => clk,
--		reset => reset,
--		inn => c_wire5,
--		ut => c_wire4
--	);
--	
--control_shift_reg3 : control_shift_reg
--	GENERIC MAP ( length => length5 )
--	PORT MAP (
--		clk => clk,
--		reset => reset,
--		inn => c_wire6,
--		ut => done
--	);
--	
--matrix_vector_multiplicator_control1 :  matrix_vector_multiplicator_control PORT MAP (
--	clk => clk,
--	reset => reset,
--	start => start,
--	empty_fifo1 => fifo1_empty,
--	empty_fifo2 => fifo2_empty,
--	size => size,
--	size1 => size1,
--	size2 => size2,
--	sel1 => c_wire9,
--	sel2 => c_wire7,
--	sel3 => c_wire8,
--	sel4 => c_wire5,
--	re_fifo1 => re_fifo1,
--	re_fifo2 => re_fifo2,
--	re_memory1 => re_memory1,
--	re_memory2 => re_memory2,
--	done => c_wire6
--);
--
--row_done <= c_wire4;
--	
--end rtl;	
--
