library IEEE;
use IEEE.STD_LOGIC_1164.all;
use ieee.std_logic_unsigned.all;

entity matrix_vector_multiplicator_control is
PORT
	(
		clk, reset, start, empty_fifo1, empty_fifo2 : in std_logic;
		size       : in std_logic_vector(12 downto 0);--12 downto 0-----------------------------------------------------------------------------
		sel1, sel2, sel3, sel4, re_fifo1, re_fifo2, re_memory1, re_memory2 : out std_logic;
		size1, size2 : in std_logic_vector(11 downto 0);
		done : out std_logic
	);
end entity matrix_vector_multiplicator_control;

architecture rtl of matrix_vector_multiplicator_control is

 signal counter1, counter2 :  std_logic_vector(11 downto 0);
 signal row_counter : std_logic_vector(12 downto 0);--12 downto 0------------------------------------------------------------------------------------
 type state_type1 is (idle, state_one, wait_state);
 type state_type2 is (idle, wait_all_finished_state);
 signal current_state, next_state : state_type2;
 signal current_state1, current_state2, next_state1, next_state2 : state_type1;
 signal section1_finished, section2_finished, all_finished, update_counter1, update_counter2, reset_counter1, reset_counter2, done_reg : std_logic;
 signal counter_small : std_logic_vector(3 downto 0);
 signal update_counter_small, reset_counter_small, update_row_counter, reset_row_counter : std_logic;
 
 begin
 done <= done_reg;
 
process(clk, reset) 
 begin
	if(reset = '1') then
		current_state1 <= idle;
		current_state2 <= idle;
		counter1 <= (OTHERS => '0');
		counter2 <= (OTHERS => '0');
   elsif(rising_edge(clk)) then
		current_state1 <= next_state1;
		current_state2 <= next_state2;
		if(update_counter1 = '1') then
			counter1 <= counter1 + 1;
		elsif(reset_counter1 = '1') then
			counter1 <= (OTHERS => '0');
		end if;
		
		if(update_counter2 = '1') then
			counter2 <= counter2 + 1;
		elsif(reset_counter2 = '1') then
			counter2 <= (OTHERS => '0');
		end if;
	end if;
 end process;
 
 process(current_state1, empty_fifo1, start, all_finished, done_reg, counter1, size1)
 begin
	case current_state1 is
		when idle =>
			sel2 <= '0';
		   section1_finished <= '0';
			re_memory1 <= '0';
			re_fifo1 <= '0';
			update_counter1 <= '0';
			reset_counter1 <= '0';
			if(start = '1') then
				next_state1 <= state_one;
			else
				next_state1 <= idle;
			end if;
			
		when state_one =>
		   section1_finished <= '0';
			reset_counter1 <= '0';
			if(empty_fifo1 = '0') then
				if(counter1 = size1 - 1) then
					re_fifo1 <= '1';
					re_memory1 <= '1';
					sel2 <= '1';
					update_counter1 <= '1';
					next_state1 <= wait_state;
				else
					sel2 <= '1';
					re_fifo1 <= '1';
					re_memory1 <= '1';
					update_counter1 <= '1';
					next_state1 <= state_one;
				end if;
			else
				sel2 <= '0';
				re_fifo1 <= '0';
				re_memory1 <= '0';
				update_counter1 <= '0';
				next_state1 <= state_one;
			end if;
			
		when wait_state =>
			sel2 <= '0';
		   re_fifo1 <= '0';
			re_memory1 <= '0';
			section1_finished <= '1';
			update_counter1 <= '0';
			reset_counter1 <= '1';
			if(done_reg = '1') then
				next_state1 <= idle;
			elsif(all_finished = '1') then
				next_state1 <= state_one;
			else
				next_state1 <= wait_state;
			end if;
		end case;
	end process;
	
process(current_state2, empty_fifo2, start, all_finished, done_reg, counter2, size2)
 begin
	case current_state2 is
		when idle =>
			sel3 <= '0';
		   section2_finished <= '0';
			re_memory2 <= '0';
			re_fifo2 <= '0';
			update_counter2 <= '0';
			reset_counter2 <= '0';
			if(start = '1') then
				next_state2 <= state_one;
			else
				next_state2 <= idle;
			end if;
			
		when state_one =>
		   section2_finished <= '0';
			reset_counter2 <= '0';
			if(empty_fifo2 = '0') then
				if(counter2 = size2 - 1) then
					sel3 <= '1';
					re_fifo2 <= '1';
					re_memory2 <= '1';
					update_counter2 <= '1';
					next_state2 <= wait_state;
				else
					sel3 <= '1';
					re_fifo2 <= '1';
					re_memory2 <= '1';
					update_counter2 <= '1';
					next_state2 <= state_one;
				end if;
			else
				sel3 <= '0';
				re_fifo2 <= '0';
				re_memory2 <= '0';
				update_counter2 <= '0';
				next_state2 <= state_one;
			end if;
			
		when wait_state =>
			sel3 <= '0';
		   re_fifo2 <= '0';
			re_memory2 <= '0';
			section2_finished <= '1';
			update_counter2 <= '0';
			reset_counter2 <= '1';
			if(done_reg = '1') then
				next_state2 <= idle;
			elsif(all_finished = '1') then
				next_state2 <= state_one;
			else
				next_state2 <= wait_state;
			end if;
		end case;
	end process;
	
process(clk, reset) 
 begin
	if(reset = '1') then
		current_state <= idle;
		counter_small <= (OTHERS => '0');
		row_counter <= (OTHERS => '0');
   elsif(rising_edge(clk)) then
		current_state <= next_state;
		if(update_counter_small = '1') then
			counter_small <= counter_small + 1;
		elsif(reset_counter_small = '1') then
			counter_small <= (OTHERS => '0');
		end if;
		if(update_row_counter = '1') then
			row_counter <= row_counter + 1;
		elsif(reset_row_counter = '1') then
			row_counter <= (OTHERS => '0');
		end if;
	end if;
 end process;
	
	process(current_state, section1_finished, section2_finished, counter_small, row_counter, size, start) 
	begin
	case current_state is
		when idle =>
			sel1 <= '0';
			sel4 <= '0';
			update_row_counter <= '0';
			update_counter_small <= '0';
			reset_counter_small <= '1';
			reset_row_counter <= '1';
			done_reg <= '0';
			all_finished <= '0';
			if(start = '1') then
				next_state <= wait_all_finished_state;
			else
				next_state <= idle;
			end if;
			
		when wait_all_finished_state =>
			reset_row_counter <= '0';
			
			if(section1_finished = '1' and section2_finished = '1') then
				if(counter_small = "1110") then --"0111"
					sel4 <= '1';
					sel1 <= '0';
					update_row_counter <= '1';
					update_counter_small <= '0';
					reset_counter_small <= '1';
					if(row_counter = size-1) then--size-1
						done_reg <= '1';
						all_finished <= '0';
						next_state <= idle;
					else
						done_reg <= '0';
						all_finished <= '1';
						next_state <= wait_all_finished_state;
					end if;
				else
					all_finished <= '0';
					done_reg <= '0';
					sel1 <= '1';
					sel4 <= '0';
					update_counter_small <= '1';
					reset_counter_small <= '0';
					update_row_counter <= '0';
					next_state <= wait_all_finished_state;
				end if;
			else
				next_state <= wait_all_finished_state;
				all_finished <= '0';
				done_reg <= '0';
				update_counter_small <= '0';
				reset_counter_small <= '0';
				update_row_counter <= '0';
				sel1 <= '0';
				sel4 <= '0';
			end if;
		end case;
	end process;
end rtl;

--library IEEE;
--use IEEE.STD_LOGIC_1164.all;
--use ieee.std_logic_unsigned.all;
--
--entity matrix_vector_multiplicator_control is
--PORT
--	(
--		clk, reset, start, empty_fifo1, empty_fifo2 : in std_logic;
--		size       : in std_logic_vector(3 downto 0);--12 downto 0-----------------------------------------------------------------------------
--		sel1, sel2, sel3, sel4, re_fifo1, re_fifo2, re_memory1, re_memory2 : out std_logic;
--		size1, size2 : in std_logic_vector(3 downto 0);
--		done : out std_logic
--	);
--end entity matrix_vector_multiplicator_control;
--
--architecture rtl of matrix_vector_multiplicator_control is
--
--signal counter1, counter2 :  std_logic_vector(3 downto 0);
--signal row_counter : std_logic_vector(3 downto 0);--12 downto 0------------------------------------------------------------------------------------
--type state_type1 is (idle, state_one, wait_state);
-- type state_type2 is (idle, state_one, state_two, state_three, state_four, state_five, state_six, state_seven);
-- signal current_state, next_state : state_type2;
-- signal current_state1, current_state2, next_state1, next_state2 : state_type1;
-- signal section1_finished, section2_finished, all_finished, update_counter1, update_counter2, reset_counter1, reset_counter2, done_reg : std_logic;
-- signal counter_small, count_to_seven : std_logic_vector(3 downto 0);
-- signal start_count_to_seven, start_counter_small, update_row_counter, reset_row_counter : std_logic;
-- 
-- --signal start_size, done_size : std_logic;
-- --signal teller : std_logic_vector(10 downto 0);
-- --signal teller_small : std_logic_vector(2 downto 0);
-- --signal size1_reg, size2_reg : std_logic_vector(3 downto 0);
-- begin
-- done <= done_reg;
--process(clk, reset) 
-- begin
--	if(reset = '1') then
--		current_state1 <= idle;
--		current_state2 <= idle;
--		counter1 <= (OTHERS => '0');
--		counter2 <= (OTHERS => '0');
--   elsif(rising_edge(clk)) then
--		current_state1 <= next_state1;
--		current_state2 <= next_state2;
--		if(update_counter1 = '1') then
--			counter1 <= counter1 + 1;
--		elsif(reset_counter1 = '1') then
--			counter1 <= (OTHERS => '0');
--		end if;
--		
--		if(update_counter2 = '1') then
--			counter2 <= counter2 + 1;
--		elsif(reset_counter2 = '1') then
--			counter2 <= (OTHERS => '0');
--		end if;
--	end if;
-- end process;
-- 
-- process(current_state1, empty_fifo1, start, all_finished, done_reg, counter1, size1)
-- begin
--	case current_state1 is
--		when idle =>
--			sel2 <= '0';
--		   section1_finished <= '0';
--			re_memory1 <= '0';
--			re_fifo1 <= '0';
--			update_counter1 <= '0';
--			reset_counter1 <= '0';
--			if(start = '1') then
--				next_state1 <= state_one;
--			else
--				next_state1 <= idle;
--			end if;
--			
--		when state_one =>
--		   section1_finished <= '0';
--			reset_counter1 <= '0';
--			if(empty_fifo1 = '0') then
--				if(counter1 = size1 - 1) then
--					re_fifo1 <= '1';
--					re_memory1 <= '1';
--					sel2 <= '1';
--					update_counter1 <= '1';
--					next_state1 <= wait_state;
--				else
--					sel2 <= '1';
--					re_fifo1 <= '1';
--					re_memory1 <= '1';
--					update_counter1 <= '1';
--					next_state1 <= state_one;
--				end if;
--			else
--				sel2 <= '0';
--				re_fifo1 <= '0';
--				re_memory1 <= '0';
--				update_counter1 <= '0';
--				next_state1 <= state_one;
--			end if;
--			
--		when wait_state =>
--			sel2 <= '0';
--		   re_fifo1 <= '0';
--			re_memory1 <= '0';
--			section1_finished <= '1';
--			update_counter1 <= '0';
--			reset_counter1 <= '1';
--			if(done_reg = '1') then
--				next_state1 <= idle;
--			elsif(all_finished = '1') then
--				next_state1 <= state_one;
--			else
--				next_state1 <= wait_state;
--			end if;
--		end case;
--	end process;
--	
--process(current_state2, empty_fifo2, start, all_finished, done_reg, counter2, size2)
-- begin
--	case current_state2 is
--		when idle =>
--			sel3 <= '0';
--		   section2_finished <= '0';
--			re_memory2 <= '0';
--			re_fifo2 <= '0';
--			update_counter2 <= '0';
--			reset_counter2 <= '0';
--			if(start = '1') then
--				next_state2 <= state_one;
--			else
--				next_state2 <= idle;
--			end if;
--			
--		when state_one =>
--		   section2_finished <= '0';
--			reset_counter2 <= '0';
--			if(empty_fifo2 = '0') then
--				if(counter2 = size2 - 1) then
--					sel3 <= '1';
--					re_fifo2 <= '1';
--					re_memory2 <= '1';
--					update_counter2 <= '1';
--					next_state2 <= wait_state;
--				else
--					sel3 <= '1';
--					re_fifo2 <= '1';
--					re_memory2 <= '1';
--					update_counter2 <= '1';
--					next_state2 <= state_one;
--				end if;
--			else
--				sel3 <= '0';
--				re_fifo2 <= '0';
--				re_memory2 <= '0';
--				update_counter2 <= '0';
--				next_state2 <= state_one;
--			end if;
--			
--		when wait_state =>
--			sel3 <= '0';
--		   re_fifo2 <= '0';
--			re_memory2 <= '0';
--			section2_finished <= '1';
--			update_counter2 <= '0';
--			reset_counter2 <= '1';
--			if(done_reg = '1') then
--				next_state2 <= idle;
--			elsif(all_finished = '1') then
--				next_state2 <= state_one;
--			else
--				next_state2 <= wait_state;
--			end if;
--		end case;
--	end process;
--	
--process(clk, reset) 
-- begin
--	if(reset = '1') then
--		current_state <= idle;
--		counter_small <= (OTHERS => '0');
--		row_counter <= (OTHERS => '0');
--   elsif(rising_edge(clk)) then
--		current_state <= next_state;
--		if(start_counter_small = '1') then
--			counter_small <= counter_small + 1;
--		else
--			counter_small <= (OTHERS => '0');
--		end if;
--		if(update_row_counter = '1') then
--			row_counter <= row_counter + 1;
--		end if;
--		if(reset_row_counter = '1') then
--			row_counter <= (OTHERS => '0');
--		end if;
--	end if;
-- end process;
--	
--	process(current_state, section1_finished, section2_finished, counter_small, size2, row_counter, size) 
--	begin
--	case current_state is
--		when idle =>
--			sel1 <= '0';
--			sel4 <= '0';
--			if(section1_finished = '1' and section2_finished = '1') then
--				update_row_counter <= '1';
--				next_state <= state_one;
--				if(counter_small + size2 > "0111") then
--					start_counter_small <= '0';
--					if(row_counter = size) then
--						done_reg <= '1';
--						all_finished <= '0';
--						reset_row_counter <= '1';
--						--start_counter_small <= '0';
--					else
--						done_reg <= '0';
--						all_finished <= '1';
--						reset_row_counter <= '0';
--						--start_counter_small <= '0';
--					end if;
--				else
--					all_finished <= '0';
--					done_reg <= '0';
--					start_counter_small <= '1';
--					reset_row_counter <= '0';
--				end if;
--			else
--				next_state <= idle;
--				all_finished <= '0';
--				done_reg <= '0';
--				start_counter_small <= '0';
--				update_row_counter <= '0';
--				reset_row_counter <= '0';
--			end if;
--			
--		when state_one =>
--			sel1 <= '1';
--			sel4 <= '0';
--			next_state <= state_two;
--			update_row_counter <= '0';
--			if(counter_small + size2 > "0111") then
--				if(row_counter = size) then
--						done_reg <= '1';
--						all_finished <= '0';
--						start_counter_small <= '0';
--						reset_row_counter <= '1';
--					else
--						done_reg <= '0';
--						all_finished <= '1';
--						start_counter_small <= '0';
--						reset_row_counter <= '0';
--					end if;
--			else
--					done_reg <= '0';
--					all_finished <= '0';
--					start_counter_small <= '1';
--					reset_row_counter <= '0';
--			end if;
--		when state_two =>
--			sel1 <= '1';
--			sel4 <= '0';
--			next_state <= state_three;
--			update_row_counter <= '0';
--			if(counter_small + size2 > "0111") then
--					if(row_counter = size) then
--						done_reg <= '1';
--						all_finished <= '0';
--						start_counter_small <= '0';
--						reset_row_counter <= '1';
--					else
--						done_reg <= '0';
--						all_finished <= '1';
--						start_counter_small <= '0';
--						reset_row_counter <= '0';
--					end if;
--			else
--					all_finished <= '0';
--					done_reg <= '0';
--					start_counter_small <= '1';
--					reset_row_counter <= '0';
--			end if;
--		when state_three =>
--			sel1 <= '1';
--			sel4 <= '0';
--			next_state <= state_four;
--			update_row_counter <= '0';
--			if(counter_small + size2 > "0111") then
--					if(row_counter = size) then
--						done_reg <= '1';
--						all_finished <= '0';
--						start_counter_small <= '0';
--						reset_row_counter <= '1';
--					else
--						done_reg <= '0';
--						all_finished <= '1';
--						start_counter_small <= '0';
--						reset_row_counter <= '0';
--					end if;
--			else
--					all_finished <= '0';
--					done_reg <= '0';
--					start_counter_small <= '1';
--					reset_row_counter <= '0';
--			end if;
--		when state_four =>
--			sel1 <= '1';
--			sel4 <= '0';
--			next_state <= state_five;
--			update_row_counter <= '0';
--			if(counter_small + size2 > "0111") then
--					if(row_counter = size) then
--						done_reg <= '1';
--						all_finished <= '0';
--						start_counter_small <= '0';
--						reset_row_counter <= '1';
--					else
--						done_reg <= '0';
--						all_finished <= '1';
--						start_counter_small <= '0';
--						reset_row_counter <= '0';
--					end if;
--			else
--					all_finished <= '0';
--					done_reg <= '0';
--					start_counter_small <= '1';
--					reset_row_counter <= '0';
--			end if;
--		when state_five =>
--			sel1 <= '1';
--			sel4 <= '0';
--			next_state <= state_six;
--			update_row_counter <= '0';
--			if(counter_small + size2 > "0111") then
--					if(row_counter = size) then
--						done_reg <= '1';
--						all_finished <= '0';
--						start_counter_small <= '0';
--						reset_row_counter <= '1';
--					else
--						done_reg <= '0';
--						all_finished <= '1';
--						start_counter_small <= '0';
--						reset_row_counter <= '0';
--					end if;
--			else
--					all_finished <= '0';
--					done_reg <= '0';
--					start_counter_small <= '1';
--					reset_row_counter <= '0';
--			end if;
--		when state_six =>
--			sel1 <= '1';
--			sel4 <= '0';
--			next_state <= state_seven;
--			update_row_counter <= '0';
--			if(counter_small + size2 > "0111") then
--					if(row_counter = size) then
--						done_reg <= '1';
--						all_finished <= '0';
--						start_counter_small <= '0';
--						reset_row_counter <= '1';
--					else
--						done_reg <= '0';
--						all_finished <= '1';
--						start_counter_small <= '0';
--						reset_row_counter <= '0';
--					end if;
--			else
--					all_finished <= '0';
--					done_reg <= '0';
--					start_counter_small <= '1';
--					reset_row_counter <= '0';
--			end if;
--		when state_seven =>
--			sel1 <= '1';
--			sel4 <= '1';
--			next_state <= idle;
--			update_row_counter <= '0';
--			if(counter_small + size2 > "0111") then
--					start_counter_small <= '0';
--					if(row_counter = size) then
--						done_reg <= '1';
--						all_finished <= '0';
--						reset_row_counter <= '1';
--						--start_counter_small <= '0';
--					else
--						done_reg <= '0';
--						all_finished <= '1';
--						reset_row_counter <= '0';
--						--start_counter_small <= '0';
--					end if;
--			else
--					all_finished <= '0';
--					done_reg <= '0';
--					start_counter_small <= '1';
--					reset_row_counter <= '0';
--			end if;
--		end case;
--	end process;
--end rtl;