library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity top is
	port
   (
      inndata_fifo1, inndata_fifo2, data_b : in std_logic_vector(63 downto 0);
		clk, fifo_clk : in std_logic;
		reset_inn, start_inn : in std_logic_vector(0 downto 0);
		size : in std_logic_vector(12 downto 0);
		wr_fifo1, wr_fifo2, wr_b, result_fifo_read: in std_logic;
		fifo1_full, fifo2_full, b_fifo_full, reset_full, start_full, result_fifo_empty, done : out std_logic;
		wrusedw1, wrusedw2 : out std_logic_vector(7 downto 0);
		result : out std_logic_vector(63 downto 0)
   );
end entity top;

architecture rtl of top is

component top_control
PORT (
	clk 		: 			IN STD_LOGIC;
	reset    :        IN STD_LOGIC;
	size     :        IN STD_LOGIC_VECTOR(12 downto 0);---------12 downto 0--------------------------------
	start, done_matrix_vector_mult, done_rr, done_dq, done_division, done_size, done_fill_memory, finish, b_fifo_empty : in std_logic;
	d_sel1, d_sel2, d_sel3, d_sel4, d_sel5, d_sel9, d_sel10, d_sel11, d_sel12, d_sel13, d_sel14, d_sel15, d_sel16 		: 	OUT STD_LOGIC;
	c_sel1, c_sel2, d_sel6, d_sel7, d_sel8 : out std_logic_vector(1 downto 0);
	c_sel3, c_sel4, c_sel5, c_sel6, c_sel7, c_sel8, c_sel9, c_sel10 : out std_logic;
	control1, control2, start_matrix_vector_mult, start_dot_product, start_size, start_fill_memory, mode_dot_product, done : out std_logic;
	error : out std_logic_vector(63 downto 0);
	iteration_counter_out : out std_logic_vector(18 downto 0);
	b_fifo_read : out std_logic
	--size_read : out std_logic;
	--size_empty : in std_logic
	--b_fifo_empty : in std_logic
);
end component;

component clock_crossing_fifo
PORT
	(
		data		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		rdclk		: IN STD_LOGIC ;
		rdreq		: IN STD_LOGIC ;
		wrclk		: IN STD_LOGIC ;
		wrreq		: IN STD_LOGIC ;
		q		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		rdempty		: OUT STD_LOGIC ;
		wrfull		: OUT STD_LOGIC 
	);
end component;

component clock_domain_fifo
	PORT
	(
		aclr		: IN STD_LOGIC  := '0';
		data		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
		rdclk		: IN STD_LOGIC ;
		rdreq		: IN STD_LOGIC ;
		wrclk		: IN STD_LOGIC ;
		wrreq		: IN STD_LOGIC ;
		q		: OUT STD_LOGIC_VECTOR (63 DOWNTO 0);
		rdempty		: OUT STD_LOGIC ;
		wrfull		: OUT STD_LOGIC 
	);
end component;

component clock_domain_fifo2
	PORT
	(
		aclr		: IN STD_LOGIC  := '0';
		data		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
		rdclk		: IN STD_LOGIC ;
		rdreq		: IN STD_LOGIC ;
		wrclk		: IN STD_LOGIC ;
		wrreq		: IN STD_LOGIC ;
		q		: OUT STD_LOGIC_VECTOR (63 DOWNTO 0);
		rdempty		: OUT STD_LOGIC ;
		wrfull		: OUT STD_LOGIC 
	);
end component;

component fifo
PORT
	(
		aclr		: IN STD_LOGIC  := '0';
		data		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
		rdclk		: IN STD_LOGIC ;
		rdreq		: IN STD_LOGIC ;
		wrclk		: IN STD_LOGIC ;
		wrreq		: IN STD_LOGIC ;
		q		: OUT STD_LOGIC_VECTOR (63 DOWNTO 0);
		rdempty		: OUT STD_LOGIC ;
		wrfull		: OUT STD_LOGIC;
		wrusedw		: OUT STD_LOGIC_VECTOR (7 DOWNTO 0)
	);
 end component;

component matrix_vector_multiplicator
port
   (
      inndata1, inndata2, inndata3, inndata4		 : in std_logic_vector(63 downto 0); 
		reset, clk, start, fifo1_empty, fifo2_empty  : in std_logic;
		size1, size2 : in std_logic_vector(11 downto 0);
		size : in std_logic_vector(12 downto 0);--12 downto 0-------------------------------------------------------------------------------------------------
		done, re_fifo1, re_fifo2, re_memory1, re_memory2, row_done : out std_logic;
		result : out std_logic_vector(63 downto 0)
   );
end component;

component int_ram_shell
    port
   (
		inndata	: in std_logic_vector(63 downto 0);
	   clk, reset : in std_logic;
		wren, rden : in std_logic;
		size  : in std_logic_vector(12 downto 0);--12 downto 0-------------------------------------------------
		utdata : out std_logic_vector(63 downto 0)
   );
  end component;
  
component int_ram_shell2
    port
   (
		inndata	: in std_logic_vector(63 downto 0);
	   clk, reset : in std_logic;
		wren, rden : in std_logic;
		size  : in std_logic_vector(11 downto 0);--11 downto 0 ------------------------------------------------
		utdata : out std_logic_vector(63 downto 0)
   );
  end component;
  
  component dot_product
	port
   (
      data_a, data_b		 : in std_logic_vector(63 downto 0); 
		reset, clk, start, mode  : in std_logic;
		size : in std_logic_vector(12 downto 0);--12 downto 0-----------------------------------------------
		done_rr : out std_logic;
		done_dq : out std_logic;
		result : out std_logic_vector(63 downto 0)
   );
  end component;
  
  component calculate_size
  port
   (
		clk, reset, start : in std_logic;
      size : in std_logic_vector(12 downto 0); 
		done : out std_logic;
      size1, size2 : out std_logic_vector(11 downto 0)
	);
	end component;
	
	component fill_memory_control
	 port
   (
		clk, reset, start : in std_logic;
      size : in std_logic_vector(12 downto 0); 
		wr_memory1, wr_memory2 : out std_logic;
		done : out std_logic
      
	);
	end component;
  
  component data_mux2
  port
   (
      a, b : in std_logic_vector(63 downto 0); 
      sel : in std_logic;
		result : out std_logic_vector(63 downto 0)
   );
	end component;
	
	component data_mux4
  port
   (
      a, b, c, d : in std_logic_vector(63 downto 0); 
      sel : in std_logic_vector(1 downto 0);
		result : out std_logic_vector(63 downto 0)
   );
	end component;
	
	component control_mux2
	port
   (
      a, b : in std_logic; 
      sel : in std_logic;
		result : out std_logic
   );
	end component;
	
	component control_mux4
	port
   (
      a, b, c, d : in std_logic; 
      sel : in std_logic_vector(1 downto 0);
		result : out std_logic
   );
	end component;
	
	component reg
	PORT (
		clk 		: 			IN STD_LOGIC;
		reset    :        IN STD_LOGIC;
		inn 		: 			IN STD_LOGIC_VECTOR(63 DOWNTO 0);
		ut 		: 			OUT STD_LOGIC_VECTOR(63 DOWNTO 0)
   );
   end component;
	
	component flip_flop
	PORT (
		clk 		: 			IN STD_LOGIC;
		reset    :        IN STD_LOGIC;
		inn 		: 			IN STD_LOGIC;
		ut 		: 			OUT STD_LOGIC
   );
   end component;
	
  component multiplier_ny
    PORT
	(
		aclr     : IN STD_LOGIC;
		clock		: IN STD_LOGIC ;
		dataa		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
		datab		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
		result		: OUT STD_LOGIC_VECTOR (63 DOWNTO 0)
	);
  end component;
  
  component multiplier
    PORT
	(
		aclr     : IN STD_LOGIC;
		clock		: IN STD_LOGIC ;
		dataa		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
		datab		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
		result		: OUT STD_LOGIC_VECTOR (63 DOWNTO 0)
	);
  end component;
  
  component divider
	PORT
	(
		aclr     : IN STD_LOGIC;
		clock		: IN STD_LOGIC ;
		dataa		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
		datab		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
		result		: OUT STD_LOGIC_VECTOR (63 DOWNTO 0)
	);
	end component;
  
  component sub_ny
  PORT
	(
		aclr     : IN STD_LOGIC;
		clock		: IN STD_LOGIC ;
		dataa		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
		datab		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
		result		: OUT STD_LOGIC_VECTOR (63 DOWNTO 0)
	);
  end component;
  
  component add_ny
  PORT
	(
		aclr     : IN STD_LOGIC;
		clock		: IN STD_LOGIC ;
		dataa		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
		datab		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
		result		: OUT STD_LOGIC_VECTOR (63 DOWNTO 0)
	);
	end component;
  
   component shift_reg
		generic (length : integer
			);
		PORT (
			clk 		: 			IN STD_LOGIC;
			inn 		: 			IN STD_LOGIC;
			tap1		:        OUT STD_LOGIC;
			ut 		: 			OUT STD_LOGIC
		);
   end component;
	
	component shift_reg_div
		generic (length : integer := 25
			);
		PORT (
			clk 		: 			IN STD_LOGIC;
			inn 		: 			IN STD_LOGIC;
			ut 		: 			OUT STD_LOGIC
		);
	end component;
	
	component control_shift_reg
		generic (length : integer := 2);
		PORT (
			clk 		: 			IN STD_LOGIC;
			reset    :        IN STD_LOGIC;
			inn 		: 			IN STD_LOGIC;
			ut 		: 			OUT STD_LOGIC
		);
	end component;
	
	component comparator
	PORT
	(
		aclr		: IN STD_LOGIC ;
		clock		: IN STD_LOGIC ;
		dataa		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
		datab		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
		alb		: OUT STD_LOGIC 
	);
	end component;
	
--	component fifo_size
--	PORT
--	(
--		data		: IN STD_LOGIC_VECTOR (10 DOWNTO 0);
--		rdclk		: IN STD_LOGIC ;
--		rdreq		: IN STD_LOGIC ;
--		wrclk		: IN STD_LOGIC ;
--		wrreq		: IN STD_LOGIC ;
--		q		: OUT STD_LOGIC_VECTOR (10 DOWNTO 0);
--		rdempty		: OUT STD_LOGIC ;
--		wrfull		: OUT STD_LOGIC 
--	);
--	end component;
--  
  
  --signals
  signal d_wire1, d_wire2, d_wire3, d_wire4, d_wire5, d_wire6, d_wire7, d_wire8, d_wire9, d_wire10, d_wire11, d_wire12 : std_logic_vector(63 downto 0);
  signal d_wire13, d_wire14, d_wire15, d_wire16, d_wire17, d_wire18, d_wire19, d_wire20, d_wire21, d_wire22, d_wire23, d_wire24, d_wire25, d_wire26 : std_logic_vector(63 downto 0);
  signal d_wire27, d_wire28, d_wire29, d_wire30, d_wire31, d_wire32, d_wire33, d_wire34, d_wire35, d_wire36, d_wire37, d_wire38, d_wire39, d_wire40, d_wire41, d_wire42, d_wire43, d_wire44 : std_logic_vector(63 downto 0);
  signal c_wire1, c_wire2, c_wire3, c_wire4, c_wire5, c_wire6, c_wire7, c_wire9, c_wire10, c_wire11, c_wire12, c_wire13, c_wire14 : std_logic;
  signal c_wire15, c_wire16, c_wire17, c_wire18, c_wire19, c_wire20, c_wire21, c_wire22, c_wire23, c_wire24, c_wire25, c_wire26, c_wire27, c_wire28, c_wire29, c_wire30: std_logic;
  signal c_wire31, c_wire32, c_wire33, c_wire34, c_wire35 : std_logic;
  signal c_sel3, c_sel4, c_sel5, c_sel6, c_sel7, c_sel8, c_sel9, c_sel10, d_sel1, d_sel2, d_sel3, d_sel4, d_sel5, d_sel9, d_sel10, d_sel11, d_sel12, d_sel13, d_sel14, d_sel15, d_sel16 : std_logic;
  signal c_sel1, c_sel2, d_sel6, d_sel7, d_sel8 : std_logic_vector(1 downto 0);
  signal control1, control2, start_dot_product, mode_dot_product, start_matrix_vector_mult, done_matrix_vector_mult : std_logic;
  signal b_fifo_read, b_fifo_empty, result_fifo_full : std_logic;
  signal s_wire1, s_wire2 : std_logic_vector(11 downto 0);
  signal reset_read, reset_empty, reset, start_read, start_empty, start : std_logic;
  signal reset_ut, start_ut : std_logic_vector(0 downto 0);
  signal iteration_counter : std_logic_vector(18 downto 0);
  --signal size : std_logic_vector(10 downto 0);
 -- signal size_read, size_empty : std_logic;
  constant length1 : integer := 27;--14
  constant length2 : integer := 24;
  constant length3 : integer := 2;
  
  begin
  
--  fifo_size1 : fifo_size port map (
--	data => size_inn,
--	rdclk => clk,
--	rdreq => size_read,
--	wrclk => fifo_clk,
--	wrreq => size_write,
--	q => size,
--	rdempty => size_empty,
--	wrfull => size_full
--  );
  
  clock_crossing_fifo_reset : clock_crossing_fifo port map (
		data		=> reset_inn,
		rdclk		=> clk,
		rdreq		=> reset_read,
		wrclk		=> fifo_clk,
		wrreq		=> reset_inn(0),
		q		   => reset_ut,
		rdempty	=> reset_empty,
		wrfull	=> reset_full
	);
	reset_read <= not reset_empty;
	--reset <= reset_ut(0);
	
	process(clk) begin
	if(rising_edge(clk)) then
		if(reset_empty = '1') then
			reset <= '0';
		else
			reset <= '1';
		end if;
	end if;
	end process;
	
	process(clk) begin
	if(rising_edge(clk)) then
		if(start_empty = '1') then
			start <= '0';
		else
			start <= '1';
		end if;
	end if;
	end process;
	
	clock_crossing_fifo_start : clock_crossing_fifo port map (
		data		=> start_inn,
		rdclk		=> clk,
		rdreq		=> start_read,
		wrclk		=> fifo_clk,
		wrreq		=> start_inn(0),
		q		   => start_ut,
		rdempty	=> start_empty,
		wrfull	=> start_full
	);
	start_read <= not start_empty;
	--start <= start_ut(0);
  
  clock_domain_fifo1 : clock_domain_fifo port map (
		aclr		=> reset,
		data		=> data_b,
		rdclk		=> clk,
		rdreq		=> b_fifo_read,
		wrclk		=> fifo_clk,
		wrreq		=> wr_b,
		q		   => d_wire43,
		rdempty	=> b_fifo_empty,
		wrfull	=> b_fifo_full
	);
	
	clock_domain_fifo_2 : clock_domain_fifo2 port map (
		aclr		=> reset,
		data		=> d_wire44,
		rdclk		=> fifo_clk,
		rdreq		=> result_fifo_read,
		wrclk		=> clk,
		wrreq		=> c_wire35, --d_sel16,
		q		   => result,
		rdempty	=> result_fifo_empty,
		wrfull	=> result_fifo_full
	);
	
	flip_flop2 : flip_flop port map (
		clk => clk,
		reset => reset,
		inn => b_fifo_read,
		ut => c_wire33
	);
	
	int_ram_b : int_ram_shell PORT MAP (
		inndata	 => d_wire43,--data_b,
		clk	 => clk,
		reset	 => reset,
		wren	 => c_wire33,
		rden => c_wire30,
		size => size,
		utdata	 => d_wire42
	);
  
  fill_memory_control1 : fill_memory_control PORT MAP (
	clk => clk,
	reset => reset,
	start => c_wire24,
	size => size,
	wr_memory1 => c_wire25,
	wr_memory2 => c_wire26,
	done => c_wire27
  );
  
  reg_result_mux : control_shift_reg 
  generic map ( length =>  2)
  port map (
	clk => clk,
	reset => reset,
	inn => c_wire29,--c_wire29,
	ut => c_wire35 --d_sel16
  );
  
  top_control1 : top_control PORT MAP (
	clk => clk,
	reset => reset,
	size => size,
	start => start,
	done_matrix_vector_mult => done_matrix_vector_mult,
	done_rr => c_wire12,
	done_dq => c_wire13,
	done_division => c_wire11,
	done_size => c_wire22,
	done_fill_memory => c_wire27,
	finish => c_wire28,
	d_sel1 => d_sel1,
	d_sel2 => d_sel2,
	d_sel3 => d_sel3,
	d_sel4 => d_sel4,
	d_sel5 => d_sel5,
	d_sel6 => d_sel6,
	d_sel7 => d_sel7,
	d_sel8 => d_sel8,
	d_sel9 => d_sel9,
	d_sel10 => d_sel10,
	d_sel11=> d_sel11,
	d_sel12 => d_sel12,
	d_sel13 => d_sel13,
	d_sel14 => d_sel14,
	d_sel15 => d_sel15,
	d_sel16 => c_wire29,
	c_sel1 => c_sel1,
	c_sel2 => c_sel2,
	c_sel3 => c_sel3,
	c_sel4 => c_sel4,
	c_sel5 => c_sel5,
	c_sel6 => c_sel6,
	c_sel7 => c_sel7,
	c_sel8 => c_sel8,
	c_sel9 => c_sel9,
	c_sel10 => c_sel10,
	control1 => control1,
	control2 => control2,
	start_matrix_vector_mult => start_matrix_vector_mult,
	start_dot_product => start_dot_product,
	start_size => c_wire23,
	start_fill_memory => c_wire24,
	mode_dot_product => mode_dot_product,
	done => done,
	error => d_wire41,
	b_fifo_read => b_fifo_read,
	b_fifo_empty => b_fifo_empty,
	iteration_counter_out => iteration_counter
	--size_read => size_read,
	--size_empty => size_empty
);

  fifo1 : fifo PORT MAP (
   aclr => reset,
	data => inndata_fifo1,
	rdclk => clk,
	rdreq => c_wire15,
	wrclk => fifo_clk, --clk,
	wrreq => wr_fifo1,
	q => d_wire27,
	rdempty => c_wire16,
	wrfull => fifo1_full,
	wrusedw => wrusedw1
);

memory1 : int_ram_shell2 PORT MAP (
	inndata => d_wire35,
	clk => clk,
	reset => reset,
	wren => c_wire25,
	rden => c_wire17,
	size => s_wire1,
	utdata => d_wire28
);

fifo2 : fifo PORT MAP (
	aclr => reset,
	data => inndata_fifo2,
	rdclk => clk,
	rdreq => c_wire18,
	wrclk => fifo_clk, --clk,
	wrreq => wr_fifo2,
	q => d_wire29,
	rdempty => c_wire19,
	wrfull => fifo2_full,
	wrusedw => wrusedw2
);

memory2 : int_ram_shell2 PORT MAP (
	inndata => d_wire35,
	clk => clk,
	reset => reset,
	wren => c_wire26,
	rden => c_wire20,
	size => s_wire2,
	utdata => d_wire30
);

matrix_vector_multiplicator1 : matrix_vector_multiplicator PORT MAP (
	inndata1 => d_wire27,
	inndata2 => d_wire28,
	inndata3 => d_wire29,
	inndata4 => d_wire30,
	reset => reset,
	clk => clk,
	start => start_matrix_vector_mult,
	fifo1_empty => c_wire16,
	fifo2_empty => c_wire19,
	size1 => s_wire1,
	size2 => s_wire2,
	size => size,
	done => done_matrix_vector_mult,
	re_fifo1 => c_wire15,
	re_fifo2 => c_wire18,
	re_memory1 => c_wire17,
	re_memory2 => c_wire20,
	row_done => c_wire14,
	result => d_wire26
);
  
  calculate_size1 : calculate_size PORT MAP (
	clk => clk,
	reset => reset,
	start => c_wire23,
	size => size,
	done => c_wire22,
	size1 => s_wire1,
	size2 => s_wire2
);
   multiplier_q : multiplier_ny PORT MAP (
		aclr => reset,
		clock => clk,
		dataa	 => d_wire32,
		datab	 => d_wire31,
		result	 => d_wire10
	);
	
	multiplier_d : multiplier_ny PORT MAP (
		aclr => reset,
		clock => clk,
		dataa	 => d_wire33,
		datab	 => d_wire31,
		result	 => d_wire6
	);
	
	add1 : add_ny PORT MAP (
		aclr => reset,
		clock => clk,
		dataa	 => d_wire7,
		datab	 => d_wire6,
		result	 => d_wire11
	);
		
	sub1 : sub_ny PORT MAP (
		aclr => reset,
		clock => clk,
		dataa	 => d_wire8,
		datab	 => d_wire10,
		result	 => d_wire12
	);
	
	div1 : divider PORT MAP (
		aclr => reset,
		clock => clk,
		dataa	 => d_wire34,--d_wire14,
		datab	 => d_wire17,
		result	 => d_wire20
	);
	
	dot_product1 : dot_product port map (
		data_a => d_wire24,
		reset => reset,
		data_b => d_wire25,
		clk => clk,
		start => start_dot_product,
		mode => mode_dot_product,
		size => size,
		done_rr => c_wire12,
		done_dq => c_wire13,
		result => d_wire23
	);
	
	d_mux1 : data_mux2 port map (
		a => d_wire11,
		b => (OTHERS => '0'),
		sel => d_sel1,
		result => d_wire1
	);
	
	d_mux2 : data_mux2 port map (
		a => d_wire12,
		b => d_wire42,--(OTHERS => '0'),
		sel => d_sel2,
		result => d_wire2
	);
	
	d_mux3 : data_mux2 port map (
		a => d_wire11,
		b => d_wire42,--(OTHERS => '0'),
		sel => d_sel3,
		result => d_wire3
	);
	
	d_mux4 : data_mux2 port map (
		a => d_wire9,
		b => d_wire8,
		sel => d_sel4,
		result => d_wire7
	);
	
	d_mux13 : data_mux2 port map (
		a => d_wire4,
		b => (OTHERS => '0'),
		sel => d_sel10,
		result => d_wire32
	);
	
	d_mux14 : data_mux2 port map (
		a => d_wire5,
		b => (OTHERS => '0'),
		sel => d_sel11,
		result => d_wire33
	);
	--DIVIDOR--------------------------------------------------------------------------------------------------------
	d_mux5 : data_mux2 port map (
		a => d_wire23,
		b => d_wire14,
		sel => c_wire12,
		result => d_wire13
	);
	
	d_mux6 : data_mux2 port map (
		a => d_wire14,
		b => d_wire16,
		sel => d_sel5,
		result => d_wire15
	);
	
	d_mux7 : data_mux2 port map (
		a => d_wire23,
		b => d_wire19,
		sel => c_wire13,
		result => d_wire18
	);
	
	d_mux8 : data_mux4 port map (
		a => d_wire16,
		b => d_wire19,
		c => x"3FF0000000000000",
		d => x"3FF0000000000000",
		sel => d_sel6,
		result => d_wire17
	);
	
	d_mux9 : data_mux2 port map (
		a => d_wire20,
		b => d_wire22,
		sel => c_wire11,
		result => d_wire21
	);
	
	
	--DOT PRODUCT-------------------------------------------------------------------------
	d_mux10 : data_mux4 port map (
		a => d_wire8,
		b => d_wire5,
		c => (OTHERS => '0'),
		d => (OTHERS => '0'),
		sel => d_sel8,
		result => d_wire24
	);
	
	d_mux11 : data_mux4 port map (
		a => d_wire8,
		b => d_wire4,
		c => (OTHERS => '0'),
		d => (OTHERS => '0'),
		sel => d_sel7,
		result => d_wire25
	);
	
	d_mux12 : data_mux2 port map (
		a => d_wire22,
		b => (OTHERS => '0'),
		sel => d_sel9,
		result => d_wire31
	);
	
	d_mux15 : data_mux2 port map (
		a => d_wire14,
		b => (OTHERS => '0'),
		sel => d_sel12,
		result => d_wire34
	);
	
	d_mux16 : data_mux2 port map (
		a => d_wire5,
		b => d_wire9,
		sel => d_sel13,
		result => d_wire35
	);
	
	d_mux17 : data_mux2 port map (
		a => d_wire14,
		b => d_wire39,
		sel => d_sel14,
		result => d_wire38
	);
	
	d_mux18 : data_mux2 port map (
		a => d_wire41,
		b => d_wire37,
		sel => d_sel15,
		result => d_wire36
	);
	
	d_mux19 : data_mux2 port map (
		a => d_wire9,
		b => (OTHERS => '0'),
		sel => c_wire35, --d_sel16,
		result => d_wire44
	);
	
	--MINUS PLUSS----------------------------------------------------------------------
	c_mux1 : control_mux4 port map (
		a => c_wire10,
		b => '0',
		c => '1',
		d => '0',
		sel => c_sel1,
		result => c_wire1
	);
	
	c_mux2 : control_mux4 port map (
		a => c_wire9,
		b => control1,
		c => '0',
		d => '0',
		sel => c_sel2,
		result => c_wire2
	);
	
	c_mux3 : control_mux2 port map (
		a => c_wire10,
		b => c_wire32,--'0',
		sel => c_sel3,
		result => c_wire3
	);
	
	c_mux4 : control_mux2 port map (
		a => c_wire9,
		b => control1,---------------------------------------------------------------
		sel => c_sel4,
		result => c_wire4
	);
	
	c_mux5 : control_mux2 port map (
		a => c_wire10,
		b => c_wire32,--'0',
		sel => c_sel5,
		result => c_wire5
	);
	
	c_mux6 : control_mux2 port map (
		a => control1,-----------------------------------------------------------------
		b => '0',
		sel => c_sel6,
		result => c_wire6
	);
	
	c_mux7 : control_mux2 port map (
		a => control1,------------------------------------------------------------------------
		b => '0',
		sel => c_sel7,
		result => c_wire7
	);
	
	c_mux8 : control_mux2 port map (
		a => control1,-----------------------------------------------------------------------
		b => '0',
		sel => c_sel8,
		result => c_wire21
	);
	
	c_mux9 : control_mux2 port map (
		a => control1,
		b => '0',
		sel => c_sel9,
		result => c_wire30
	);
	
	c_mux10 : control_mux2 port map (
		a => control1,
		b => '0',
		sel => c_sel10,
		result => c_wire31
	);
	--DIVIDOR---------------------------------------------------------------------
	reg_dtq : reg port map (
		clk => clk,
		reset => reset,
		inn => d_wire18,
		ut => d_wire19
	);
	
	reg_d_new : reg port map (
		clk => clk,
		reset => reset,
		inn => d_wire13,
		ut => d_wire14
	);
	
	reg_d_old : reg port map (
		clk => clk,
		reset => reset,
		inn => d_wire15,
		ut => d_wire16
	);
	
	reg_alfa_beta : reg port map (
		clk => clk,
		reset => reset,
		inn => d_wire21,
		ut => d_wire22
	);
	
	reg_first_d_new : reg port map (
		clk => clk,
		reset => reset,
		inn => d_wire38,
		ut => d_wire39
	);
	
	reg_error : reg port map (
		clk => clk,
		reset => reset,
		inn => d_wire36,
		ut => d_wire37
	);
	
	error_multiplier : multiplier port map (
		aclr => reset,
		clock => clk,
		dataa => d_wire37,
		datab => d_wire39,
		result => d_wire40
	);
	
	comparator1 : comparator port map (
		aclr => reset,
		clock => clk,
		dataa => d_wire14,
		datab => d_wire40,
		alb => c_wire28
	);
	
	int_ram_q : int_ram_shell PORT MAP (
		inndata	 => d_wire26,
		clk	 => clk,
		reset	 => reset,
		wren	 => c_wire14,
		rden => c_wire7,
		size => size,
		utdata	 => d_wire4
	);

	int_ram_r : int_ram_shell PORT MAP (
		inndata	 => d_wire2,
		clk	 => clk,
		reset	 => reset,
		wren	 => c_wire3,
		rden => c_wire4,
		size => size,
		utdata	 => d_wire8
	);
	
	int_ram_x : int_ram_shell PORT MAP (
		inndata	 => d_wire1,
		clk	 => clk,
		reset	 => reset,
		wren	 => c_wire1,
		rden => c_wire2,
		size => size,
		utdata	 => d_wire9
	);
	
	int_ram_d : int_ram_shell PORT MAP (
		inndata	 => d_wire3,
		clk	 => clk,
		reset	 => reset,
		wren	 => c_wire5,
		rden => c_wire6,
		size => size,
		utdata	 => d_wire5
	);
	
	shift_reg_b : control_shift_reg
	GENERIC MAP (length => length3)
   PORT MAP (
		clk => clk,
		reset => reset,
		inn => c_wire31,
		ut	 => c_wire32
	);
	
	shift_reg_mult1 : shift_reg
  GENERIC MAP (length => length1)
  PORT MAP (
		clk => clk,
		inn => c_wire21,
		tap1 => c_wire9,
		ut	 => c_wire10
	);
	
	shift_reg_div1 : shift_reg_div
	Generic map ( length =>  length2)
	port map (
		clk => clk,
		inn => control2,----------------------------------------------------------------------------------------
		ut => c_wire11
	);
	
end rtl;
