library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity top_level is
	port
   (
      inndata		 : in std_logic_vector(63 downto 0); 
		reset, clk, control1, control2, c_sel1, c_sel2, c_sel3, c_sel4, c_sel5, c_sel6, c_sel7, d_sel1, d_sel2, d_sel3, d_sel4, d_sel5, d_sel8, d_sel7, start, mode : in std_logic;
		d_sel6 : in std_logic_vector(1 downto 0);
		size : in std_logic_vector(3 downto 0);
		output : out std_logic_vector(63 downto 0)
   );
end entity top_level;

architecture rtl of top_level is

component int_ram_shell
    port
   (
		inndata	: in std_logic_vector(63 downto 0);
	   clk, reset : in std_logic;
		wren, rden : in std_logic;
		size  : in std_logic_vector(3 downto 0);
		utdata : out std_logic_vector(63 downto 0)
   );
  end component;
  
  component dot_product
	port
   (
      data_a, data_b		 : in std_logic_vector(63 downto 0); 
		reset, clk, start, mode  : in std_logic;
		size : in std_logic_vector(3 downto 0);
		done_rr : out std_logic;
		done_dq : out std_logic;
		result : out std_logic_vector(63 downto 0)
   );
  end component;
  
  component data_mux2
  port
   (
      a, b : in std_logic_vector(63 downto 0); 
      sel : in std_logic;
		result : out std_logic_vector(63 downto 0)
   );
	end component;
	
	component data_mux4
  port
   (
      a, b, c, d : in std_logic_vector(63 downto 0); 
      sel : in std_logic_vector(1 downto 0);
		result : out std_logic_vector(63 downto 0)
   );
	end component;
	
	component control_mux2
	port
   (
      a, b : in std_logic; 
      sel : in std_logic;
		result : out std_logic
   );
	end component;
	
	component reg
	PORT (
		clk 		: 			IN STD_LOGIC;
		reset    :        IN STD_LOGIC;
		inn 		: 			IN STD_LOGIC_VECTOR(63 DOWNTO 0);
		ut 		: 			OUT STD_LOGIC_VECTOR(63 DOWNTO 0)
   );
   end component;
	
  component multiplier
    PORT
	(
		clock		: IN STD_LOGIC ;
		dataa		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
		datab		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
		result		: OUT STD_LOGIC_VECTOR (63 DOWNTO 0)
	);
  end component;
  
  component divider
	PORT
	(
		clock		: IN STD_LOGIC ;
		dataa		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
		datab		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
		result		: OUT STD_LOGIC_VECTOR (63 DOWNTO 0)
	);
	end component;
  
  component sub
  PORT
	(
		clock		: IN STD_LOGIC ;
		dataa		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
		datab		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
		result		: OUT STD_LOGIC_VECTOR (63 DOWNTO 0)
	);
  end component;
  
  component add
  PORT
	(
		clock		: IN STD_LOGIC ;
		dataa		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
		datab		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
		result		: OUT STD_LOGIC_VECTOR (63 DOWNTO 0)
	);
	end component;
  
   component shift_reg
		generic (length : integer
			);
		PORT (
			clk 		: 			IN STD_LOGIC;
			inn 		: 			IN STD_LOGIC;
			tap1		:        OUT STD_LOGIC;
			ut 		: 			OUT STD_LOGIC
		);
   end component;
	
	component shift_reg_div
		generic (length : integer := 25
			);
		PORT (
			clk 		: 			IN STD_LOGIC;
			inn 		: 			IN STD_LOGIC;
			ut 		: 			OUT STD_LOGIC
		);
	end component;
  
  
  --signals
  signal d_wire1, d_wire2, d_wire3, d_wire4, d_wire5, d_wire6, d_wire7, d_wire8, d_wire9, d_wire10, d_wire11, d_wire12 : std_logic_vector(63 downto 0);
  signal d_wire13, d_wire14, d_wire15, d_wire16, d_wire17, d_wire18, d_wire19, d_wire20, d_wire21, d_wire22, d_wire23, d_wire24, d_wire25 : std_logic_vector(63 downto 0);
  signal c_wire1, c_wire2, c_wire3, c_wire4, c_wire5, c_wire6, c_wire7, c_wire9, c_wire10, c_wire11, c_wire12, c_wire13 : std_logic;
  --signal c_sel1, c_sel2, c_sel3, c_sel4, c_sel5, c_sel6, c_sel7 : std_logic; 
 -- signal d_sel1, d_sel2, d_sel3, d_sel4 : std_logic; 
  
  constant length1 : integer := 14;
  constant length2 : integer := 25;
  
  begin
  
   multiplier_q : multiplier PORT MAP (
		clock => clk,
		dataa	 => d_wire4,
		datab	 => d_wire22,
		result	 => d_wire10
	);
	
	multiplier_d : multiplier PORT MAP (
		clock => clk,
		dataa	 => d_wire5,
		datab	 => d_wire22,
		result	 => d_wire6
	);
	
	add1 : add PORT MAP (
		clock => clk,
		dataa	 => d_wire7,
		datab	 => d_wire6,
		result	 => d_wire11
	);
		
	sub1 : sub PORT MAP (
		clock => clk,
		dataa	 => d_wire8,
		datab	 => d_wire10,
		result	 => d_wire12
	);
	
	div1 : divider PORT MAP (
		clock => clk,
		dataa	 => d_wire14,
		datab	 => d_wire17,
		result	 => d_wire20
	);
	
	dot_product1 : dot_product port map (
	data_a => d_wire24,
	data_b => d_wire25,
	reset => reset,
	clk => clk,
	start => start,
	mode => mode,
	size => size,
	done_rr => c_wire12,
	done_dq => c_wire13,
	result => d_wire23
  );
	
	d_mux1 : data_mux2 port map (
		a => d_wire11,
		b => (OTHERS => '0'),
		sel => d_sel1,
		result => d_wire1
	);
	
	d_mux2 : data_mux2 port map (
		a => d_wire12,
		b => (OTHERS => '0'),
		sel => d_sel2,
		result => d_wire2
	);
	
	d_mux3 : data_mux2 port map (
		a => d_wire11,
		b => (OTHERS => '0'),
		sel => d_sel3,
		result => d_wire3
	);
	
	d_mux4 : data_mux2 port map (
		a => d_wire9,
		b => d_wire8,
		sel => d_sel4,
		result => d_wire7
	);
	
	d_mux5 : data_mux2 port map (
		a => d_wire23,
		b => d_wire14,
		sel => c_wire12,
		result => d_wire13
	);
	
	d_mux6 : data_mux2 port map (
		a => d_wire14,
		b => d_wire16,
		sel => d_sel5,
		result => d_wire15
	);
	
	d_mux7 : data_mux2 port map (
		a => d_wire23,
		b => d_wire19,
		sel => c_wire13,
		result => d_wire18
	);
	
	d_mux8 : data_mux4 port map (
		a => d_wire16,
		b => d_wire19,
		c => x"3FF0000000000000",
		d => x"3FF0000000000000",
		sel => d_sel6,
		result => d_wire17
	);
	
	d_mux9 : data_mux2 port map (
		a => d_wire20,
		b => d_wire22,
		sel => c_wire11,
		result => d_wire21
	);
	
	d_mux10 : data_mux2 port map (
		a => d_wire8,
		b => d_wire5,
		sel => d_sel8,
		result => d_wire24
	);
	
	d_mux11 : data_mux2 port map (
		a => d_wire8,
		b => d_wire4,
		sel => d_sel7,
		result => d_wire25
	);
	
	c_mux1 : control_mux2 port map (
		a => c_wire10,
		b => '0',
		sel => c_sel1,
		result => c_wire1
	);
	
	c_mux2 : control_mux2 port map (
		a => c_wire9,
		b => '0',
		sel => c_sel2,
		result => c_wire2
	);
	
	c_mux3 : control_mux2 port map (
		a => c_wire10,
		b => '0',
		sel => c_sel3,
		result => c_wire3
	);
	
	c_mux4 : control_mux2 port map (
		a => c_wire9,
		b => control1,
		sel => c_sel4,
		result => c_wire4
	);
	
	c_mux5 : control_mux2 port map (
		a => c_wire10,
		b => '0',
		sel => c_sel5,
		result => c_wire5
	);
	
	c_mux6 : control_mux2 port map (
		a => control1,
		b => '0',
		sel => c_sel6,
		result => c_wire6
	);
	
	c_mux7 : control_mux2 port map (
		a => control1,
		b => '0',
		sel => c_sel7,
		result => c_wire7
	);
	
	reg_dtq : reg port map (
		clk => clk,
		reset => reset,
		inn => d_wire18,
		ut => d_wire19
	);
	
	reg_d_new : reg port map (
		clk => clk,
		reset => reset,
		inn => d_wire13,
		ut => d_wire14
	);
	
	reg_d_old : reg port map (
		clk => clk,
		reset => reset,
		inn => d_wire15,
		ut => d_wire16
	);
	
	reg_alfa_beta : reg port map (
		clk => clk,
		reset => reset,
		inn => d_wire21,
		ut => d_wire22
	);
	
	int_ram_q : int_ram_shell PORT MAP (
		inndata	 => x"3FF0000000000000",--dfghjjjjjjjjjjjjjjjjjjjjjjjjjj
		clk	 => clk,
		reset	 => reset,
		wren	 => '0',--fgggggggggggggggggggggggg
		rden => c_wire7,
		size => size,
		utdata	 => d_wire4
	);

	int_ram_r : int_ram_shell PORT MAP (
		inndata	 => d_wire2,
		clk	 => clk,
		reset	 => reset,
		wren	 => c_wire3,
		rden => c_wire4,
		size => size,
		utdata	 => d_wire8
	);
	
	int_ram_x : int_ram_shell PORT MAP (
		inndata	 => d_wire1,
		clk	 => clk,
		reset	 => reset,
		wren	 => c_wire1,
		rden => c_wire2,
		size => size,
		utdata	 => d_wire9
	);
	
	int_ram_d : int_ram_shell PORT MAP (
		inndata	 => d_wire3,
		clk	 => clk,
		reset	 => reset,
		wren	 => c_wire5,
		rden => c_wire6,
		size => size,
		utdata	 => d_wire5
	);
	
	shift_reg_mult1 : shift_reg
  GENERIC MAP (length => length1)
  PORT MAP (
		clk => clk,
		inn => control1,
		tap1 => c_wire9,
		ut	 => c_wire10
	);
	
	shift_reg_div1 : shift_reg_div
	Generic map ( length =>  length2)
	port map (
		clk => clk,
		inn => control2,
		ut => c_wire11
	);
	
	output <= d_wire23;
	
end rtl;
