library altera;
use altera.altera_europa_support_lib.all;

library altera_mf;
use altera_mf.all;

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity top_top is
	port
   (
		reset_inn, inn_clk, top_clk, start_inn : in std_logic;
		done_ut  : out std_logic;
		mem_addr : OUT STD_LOGIC_VECTOR (12 DOWNTO 0);
      mem_ba : OUT STD_LOGIC_VECTOR (2 DOWNTO 0);
      mem_cas_n : OUT STD_LOGIC;
      mem_cke : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
      mem_clk : INOUT STD_LOGIC_VECTOR (0 DOWNTO 0);
      mem_clk_n : INOUT STD_LOGIC_VECTOR (0 DOWNTO 0);
      mem_cs_n : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
      mem_dm : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
      mem_dq : INOUT STD_LOGIC_VECTOR (15 DOWNTO 0);
      mem_dqs : INOUT STD_LOGIC_VECTOR (1 DOWNTO 0);
      mem_dqsn : INOUT STD_LOGIC_VECTOR (1 DOWNTO 0);
      mem_odt : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
      mem_ras_n : OUT STD_LOGIC;
      mem_reset_n : OUT STD_LOGIC;
      mem_we_n : OUT STD_LOGIC
   );
end entity top_top;

architecture rtl of top_top is

--component pll
--PORT
--	(
--		areset		: IN STD_LOGIC  := '0';
--		inclk0		: IN STD_LOGIC  := '0';
--		c0		: OUT STD_LOGIC 
--	);
--end component;

component top
port
   (
      inndata_fifo1, inndata_fifo2, data_b : in std_logic_vector(63 downto 0);
		reset_inn, clk, fifo_clk, start_inn : in std_logic;
		size : in std_logic_vector(12 downto 0);
		wr_fifo1, wr_fifo2, wr_b, result_fifo_read : in std_logic;
		fifo1_full, fifo2_full, b_fifo_full, reset_full, start_full, result_fifo_empty, done : out std_logic;
		wrusedw1, wrusedw2 : out std_logic_vector(7 downto 0);
		result : out std_logic_vector(63 downto 0)
   );
end component;

 component data_mux2
  port
   (
      a, b : in std_logic_vector(63 downto 0); 
      sel : in std_logic;
		result : out std_logic_vector(63 downto 0)
   );
	end component;
	
	component control_mux2
	port
   (
      a, b : in std_logic; 
      sel : in std_logic;
		result : out std_logic
   );
	end component;

component jtag_fifo
PORT
	(
		aclr		: IN STD_LOGIC  := '0';
		data		: IN STD_LOGIC_VECTOR (63 DOWNTO 0);
		rdclk		: IN STD_LOGIC ;
		rdreq		: IN STD_LOGIC ;
		wrclk		: IN STD_LOGIC ;
		wrreq		: IN STD_LOGIC ;
		q		: OUT STD_LOGIC_VECTOR (63 DOWNTO 0);
		rdempty		: OUT STD_LOGIC ;
		wrfull		: OUT STD_LOGIC;
		wrusedw		: OUT STD_LOGIC_VECTOR (10 DOWNTO 0)
	);
 end component;
  
--  component control_shift_reg
--		generic (length : integer := 2);
--		PORT (
--			clk 		: 			IN STD_LOGIC;
--			reset    :        IN STD_LOGIC;
--			inn 		: 			IN STD_LOGIC;
--			ut 		: 			OUT STD_LOGIC
--		);
--	end component;
	
	component virtual_jtag
	PORT
	(
		ir_out		: IN STD_LOGIC_VECTOR (1 DOWNTO 0);
		tdo		: IN STD_LOGIC ;
		ir_in		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		tck		: OUT STD_LOGIC ;
		tdi		: OUT STD_LOGIC ;
		virtual_state_cdr		: OUT STD_LOGIC ;
		virtual_state_cir		: OUT STD_LOGIC ;
		virtual_state_e1dr		: OUT STD_LOGIC ;
		virtual_state_e2dr		: OUT STD_LOGIC ;
		virtual_state_pdr		: OUT STD_LOGIC ;
		virtual_state_sdr		: OUT STD_LOGIC ;
		virtual_state_udr		: OUT STD_LOGIC ;
		virtual_state_uir		: OUT STD_LOGIC 
	);
	end component;
	
	 component ddr3_memory PORT (
        signal mem_cke : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
        signal mem_dqsn : INOUT STD_LOGIC_VECTOR (1 DOWNTO 0);
        signal mem_addr : OUT STD_LOGIC_VECTOR (12 DOWNTO 0);
        signal mem_dm : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
        signal aux_full_rate_clk : OUT STD_LOGIC;
        signal local_init_done : OUT STD_LOGIC;
        signal mem_dq : INOUT STD_LOGIC_VECTOR (15 DOWNTO 0);
        signal mem_ras_n : OUT STD_LOGIC;
        signal mem_cs_n : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
        signal phy_clk : OUT STD_LOGIC;
        signal mem_ba : OUT STD_LOGIC_VECTOR (2 DOWNTO 0);
        signal mem_reset_n : OUT STD_LOGIC;
        signal dqs_delay_ctrl_export : OUT STD_LOGIC_VECTOR (5 DOWNTO 0);
        signal aux_half_rate_clk : OUT STD_LOGIC;
        signal local_rdata : OUT STD_LOGIC_VECTOR (63 DOWNTO 0);
        signal local_rdata_valid : OUT STD_LOGIC;
        signal mem_we_n : OUT STD_LOGIC;
        signal reset_phy_clk_n : OUT STD_LOGIC;
        signal local_ready : OUT STD_LOGIC;
        signal mem_odt : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
        signal mem_cas_n : OUT STD_LOGIC;
        signal dll_reference_clk : OUT STD_LOGIC;
        signal mem_clk : INOUT STD_LOGIC_VECTOR (0 DOWNTO 0);
        signal local_refresh_ack : OUT STD_LOGIC;
        signal local_wdata_req : OUT STD_LOGIC;
        signal reset_request_n : OUT STD_LOGIC;
        signal mem_dqs : INOUT STD_LOGIC_VECTOR (1 DOWNTO 0);
        signal mem_clk_n : INOUT STD_LOGIC_VECTOR (0 DOWNTO 0);
        signal local_size : IN STD_LOGIC_VECTOR (1 DOWNTO 0);
        signal local_wdata : IN STD_LOGIC_VECTOR (63 DOWNTO 0);
        signal soft_reset_n : IN STD_LOGIC;
        signal global_reset_n : IN STD_LOGIC;
        signal local_write_req : IN STD_LOGIC;
        signal local_address : IN STD_LOGIC_VECTOR (23 DOWNTO 0);
        signal pll_ref_clk : IN STD_LOGIC;
        signal local_be : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
        signal local_read_req : IN STD_LOGIC
      );
  end component ddr3_memory;
  
   component custom_driver port (
              -- inputs:
                 signal clk : IN STD_LOGIC;
                 signal local_rdata_valid : IN STD_LOGIC;
                 signal local_ready : IN STD_LOGIC;
                 signal reset_n : IN STD_LOGIC;
					  signal local_wdata_req : IN STD_LOGIC;
					  signal jtag_fifo_empty : IN STD_LOGIC;
					  signal fifo1_full : IN STD_LOGIC;
					  signal fifo2_full : IN STD_LOGIC;
					  signal b_fifo_full : IN STD_LOGIC;
					  signal done : IN STD_LOGIC;
					  signal size : IN STD_LOGIC_VECTOR(12 DOWNTO 0);
					  signal start : IN STD_LOGIC;
              -- outputs:
                 signal local_bank_addr : OUT STD_LOGIC_VECTOR (2 DOWNTO 0);
                 signal local_be : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
                 signal local_col_addr : OUT STD_LOGIC_VECTOR (9 DOWNTO 0);
                 signal local_cs_addr : OUT STD_LOGIC;
                 signal local_read_req : OUT STD_LOGIC;
                 signal local_row_addr : OUT STD_LOGIC_VECTOR (12 DOWNTO 0);
                 signal local_size : OUT STD_LOGIC_VECTOR(1 downto 0);
                 signal local_write_req : OUT STD_LOGIC;
					  signal fifo1_write : OUT STD_LOGIC;
					  signal fifo2_write : OUT STD_LOGIC;
					  signal wr_b : OUT STD_LOGIC;
					  signal jtag_fifo_read : OUT STD_LOGIC;
					  signal items_in_fifo1 : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
					  signal items_in_fifo2 : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
					  signal write_jtag_fifo : OUT STD_LOGIC;
					  signal sel_control_mux : OUT STD_LOGIC;
					  signal sel_data_mux : OUT STD_LOGIC;
					  signal ferdig : OUT STD_LOGIC;
					  signal sel_control_mux2 : OUT STD_LOGIC;
					  signal store_size : OUT STD_LOGIC;
					  signal result_fifo_empty : IN STD_LOGIC;
					  signal result_fifo_read : OUT STD_LOGIC
					 
              );
  end component custom_driver;
	
	--FIFO Status lines
	signal rdempty_sig		: std_logic;
	signal read_en          : std_logic;
	signal read_en_pulse		: std_logic;
	signal read_en_pulse_reg	: std_logic;
	signal wrfull_sig		: std_logic;
	signal wrusedw_sig		: std_logic_vector(10 downto 0);
	
	signal write_en 			: std_logic;
	signal write_en_pulse		: std_logic;
	signal write_en_pulse_reg	: std_logic;
	signal probe_temp			: std_logic_vector(9 downto 0);
	
	signal q_temp				: std_logic_vector(63 downto 0);
	signal reset_jtag : std_logic;
	
	signal Control_reg   	: std_logic_vector(1 downto 0);
	constant PUSH			: std_logic_vector(1 downto 0) := "01";
	constant POP			: std_logic_vector(1 downto 0) := "10";
	constant RESET_CODE		: std_logic_vector(1 downto 0) := "00";
	constant NORMAL			: std_logic_vector(1 downto 0) := "11";
	
	-- JSM state decoding signals/JTAG specific signals
	
	signal update_dr   		: std_logic;
	signal update_ir		: std_logic;
	signal shift_dr			: std_logic;
	signal capture_dr       : std_logic;
	signal tdi				: std_logic;
	signal tdo				: std_logic;
	signal tck				: std_logic;
	
	signal virtual_state_cir_sig   : std_logic;
	signal virtual_state_e1dr_sig  : std_logic;
	signal virtual_state_e2dr_sig  : std_logic;
	signal virtual_state_pdr_sig   : std_logic;
	signal flush_read				: std_logic;
	
	----------------
	-- VJI shift register bank
	signal push_in			: std_logic_vector(63 downto 0);
	signal push_out			: std_logic_vector(63 downto 0);
	
	signal d_wire1, d_wire2, d_wire3, result_wire, fifo_data : std_logic_vector(63 downto 0);
	signal c_wire1, c_wire2, c_wire3, c_wire4, c_wire5, c_wire6, c_wire7, c_wire8, c_wire9, c_wire10, clk, start, done, reset : std_logic;
	signal b_wire1, b_wire2 : std_logic_vector(7 downto 0);
	signal size1 : std_logic_vector(12 downto 0);
	signal size2 : std_logic_vector(7 downto 0);
	signal done_cg : std_logic;
	signal wr_b : std_logic;
	signal write_jtag_fifo, sel_data_mux, sel_control_mux, sel_control_mux2, write_req_fifo, read_req_fifo, ferdig, store_size : std_logic;
	signal cg_clk, b_fifo_full, result_fifo_empty, result_fifo_read, reset_full, start_full: std_logic;
	
	--ddr signal-------------------------------------------------------------------------------------------------
	signal cs_n :  STD_LOGIC_VECTOR (0 DOWNTO 0);
                signal dll_reference_clk_sig :  STD_LOGIC;
                signal dqs_delay_ctrl_export_sig :  STD_LOGIC_VECTOR (5 DOWNTO 0);
                signal internal_mem_addr :  STD_LOGIC_VECTOR (12 DOWNTO 0);
                signal internal_mem_ba :  STD_LOGIC_VECTOR (2 DOWNTO 0);
                signal internal_mem_cas_n :  STD_LOGIC;
                signal internal_mem_cke :  STD_LOGIC_VECTOR (0 DOWNTO 0);
                signal internal_mem_dm :  STD_LOGIC_VECTOR (1 DOWNTO 0);
                signal internal_mem_odt :  STD_LOGIC_VECTOR (0 DOWNTO 0);
                signal internal_mem_ras_n :  STD_LOGIC;
                signal internal_mem_reset_n :  STD_LOGIC;
                signal internal_mem_we_n :  STD_LOGIC;
                signal internal_pnf :  STD_LOGIC;
                signal internal_pnf_per_byte :  STD_LOGIC_VECTOR (7 DOWNTO 0);
                signal internal_test_complete :  STD_LOGIC;
                signal internal_test_status :  STD_LOGIC_VECTOR (7 DOWNTO 0);
                signal mem_aux_full_rate_clk :  STD_LOGIC;
                signal mem_aux_half_rate_clk :  STD_LOGIC;
                signal mem_local_addr :  STD_LOGIC_VECTOR (23 DOWNTO 0);
                signal mem_local_be :  STD_LOGIC_VECTOR (7 DOWNTO 0);
                signal mem_local_col_addr :  STD_LOGIC_VECTOR (9 DOWNTO 0);
                signal mem_local_cs_addr :  STD_LOGIC;
                signal mem_local_rdata :  STD_LOGIC_VECTOR (63 DOWNTO 0);
                signal mem_local_rdata_valid :  STD_LOGIC;
                signal mem_local_read_req :  STD_LOGIC;
                signal mem_local_ready :  STD_LOGIC;
                signal mem_local_size :  STD_LOGIC_VECTOR (1 DOWNTO 0);
                signal mem_local_wdata :  STD_LOGIC_VECTOR (63 DOWNTO 0);
                signal mem_local_wdata_req :  STD_LOGIC;
                signal mem_local_write_req :  STD_LOGIC;
                signal phy_clk :  STD_LOGIC;
                signal reset_phy_clk_n :  STD_LOGIC;
                signal tie_high :  STD_LOGIC;
                signal tie_low :  STD_LOGIC;
--ddr signal slutt----------------------------------------------------------------------------------------------

	begin
	
	--ddr--------------------------------------------------------------------------------------------------------
	mem_cs_n <= cs_n;
	tie_high <= std_logic'('1');
   tie_low <= std_logic'('0');
	
	ddr3_memory_inst : ddr3_memory
    port map(
            aux_full_rate_clk => mem_aux_full_rate_clk,
            aux_half_rate_clk => mem_aux_half_rate_clk,
            dll_reference_clk => dll_reference_clk_sig,
            dqs_delay_ctrl_export => dqs_delay_ctrl_export_sig,
            global_reset_n => reset_inn,
            local_address => mem_local_addr,
            local_be => mem_local_be,
            local_init_done => open,
            local_rdata => d_wire1,
            local_rdata_valid => mem_local_rdata_valid,
            local_read_req => mem_local_read_req,
            local_ready => mem_local_ready,
            local_refresh_ack => open,
            local_size => mem_local_size,
            local_wdata => q_temp,
            local_wdata_req => mem_local_wdata_req,
            local_write_req => mem_local_write_req,
            mem_addr => internal_mem_addr(12 DOWNTO 0),
            mem_ba => internal_mem_ba,
            mem_cas_n => internal_mem_cas_n,
            mem_cke(0) => internal_mem_cke(0),
            mem_clk(0) => mem_clk(0),
            mem_clk_n(0) => mem_clk_n(0),
            mem_cs_n(0) => cs_n(0),
            mem_dm => internal_mem_dm(1 DOWNTO 0),
            mem_dq => mem_dq,
            mem_dqs => mem_dqs(1 DOWNTO 0),
            mem_dqsn => mem_dqsn(1 DOWNTO 0),
            mem_odt(0) => internal_mem_odt(0),
            mem_ras_n => internal_mem_ras_n,
            mem_reset_n => internal_mem_reset_n,
            mem_we_n => internal_mem_we_n,
            phy_clk => phy_clk,
            pll_ref_clk => inn_clk,--clock_source,
            reset_phy_clk_n => reset_phy_clk_n,
            reset_request_n => open,
            soft_reset_n => tie_high
    );
	 
	 mem_local_addr(7 DOWNTO 0) <= mem_local_col_addr(9 DOWNTO 2);
	 
	 driver : custom_driver
    port map(
				done => done_cg,
            clk => phy_clk,
            local_bank_addr => mem_local_addr(23 DOWNTO 21),
            local_be => mem_local_be,
            local_col_addr => mem_local_col_addr,
            local_cs_addr => mem_local_cs_addr,
            local_rdata_valid => mem_local_rdata_valid,
            local_read_req => mem_local_read_req,
            local_ready => mem_local_ready,
            local_row_addr => mem_local_addr(20 DOWNTO 8),
            local_size => mem_local_size,
            local_wdata_req => mem_local_wdata_req,
            local_write_req => mem_local_write_req,
				jtag_fifo_empty => rdempty_sig,
			   fifo1_full => c_wire6,
			   fifo2_full => c_wire7,
				b_fifo_full => b_fifo_full,
			   size => size1,
            fifo1_write => c_wire5,
			   fifo2_write => c_wire3,
			   jtag_fifo_read => c_wire8,
				wr_b => wr_b,
            reset_n => reset_phy_clk_n,
				items_in_fifo1 => b_wire1,
				items_in_fifo2 => b_wire2,
				write_jtag_fifo => write_jtag_fifo,
				sel_control_mux => sel_control_mux,
				sel_data_mux => sel_data_mux,
				ferdig => ferdig,
				sel_control_mux2 => sel_control_mux2,
				store_size => store_size,
				result_fifo_empty => result_fifo_empty,
				result_fifo_read => result_fifo_read,
				start => start
    );
	 
	 --vhdl renameroo for output signals
  mem_addr <= internal_mem_addr;
  --vhdl renameroo for output signals
  mem_ba <= internal_mem_ba;
  --vhdl renameroo for output signals
  mem_cas_n <= internal_mem_cas_n;
  --vhdl renameroo for output signals
  mem_cke <= internal_mem_cke;
  --vhdl renameroo for output signals
  mem_dm <= internal_mem_dm;
  --vhdl renameroo for output signals
  mem_odt <= internal_mem_odt;
  --vhdl renameroo for output signals
  mem_ras_n <= internal_mem_ras_n;
  --vhdl renameroo for output signals
  mem_reset_n <= internal_mem_reset_n;
  --vhdl renameroo for output signals
  mem_we_n <= internal_mem_we_n;
 
  --ddr slutt--------------------------------------------------------------------------------------------------
  
	
	--size1 <= "0000000001010";--"01001011000"; --1400
	start <= not start_inn;
	reset <= not reset_inn;
	done_ut <= not done;
	reset_jtag <= '1' when Control_reg = RESET_CODE and update_ir = '1' ELSE '0';
	
	process(phy_clk, reset)
	begin
		if(reset = '1') then
			size1 <= (OTHERS => '0');
		elsif(rising_edge(phy_clk)) then
			if(store_size = '1') then
				size1 <= q_temp(12 downto 0);
			end if;
		end if;
	end process;
	
	
	process(tck)    
	begin
		--if reset = '1' then
		--	push_in <= (others => '0');
		if tck'event and tck = '1' then
			if Control_reg = PUSH and shift_dr = '1' then
				push_in <= tdi & push_in(63 downto 1);
			end if;
		end if;
	end process;
	
	
	process(phy_clk,reset_jtag)
	begin
		if reset_jtag = '1' then
			write_en <= '0';
		elsif phy_clk'event and phy_clk = '1' then
			if Control_reg = PUSH and update_dr = '1' then
				write_en <= '1';
			else 
				write_en <= '0';
			end if;
		end if;
	end process;
	
	
	process(phy_clk,reset_jtag)
	begin
		if reset_jtag = '1' then
			write_en_pulse <= '0';
			write_en_pulse_reg <= '0';
		elsif phy_clk'event and phy_clk = '1' then
			write_en_pulse <= write_en;
			write_en_pulse_reg <= write_en and not write_en_pulse;
		end if;
	end process;
	
	----------------------------------------------------------------
-- Read Pulse generatation
----------------------------------------------------------------


	process(tck)
	begin
		--if reset = '1' then
		--	push_out <= (others => '0');
		if tck'event and tck = '1' then
			if Control_reg = POP and capture_dr = '1' then
				push_out <= q_temp;
			elsif Control_reg = POP and shift_dr = '1' then
				push_out <= tdi & push_out(63 downto 1);
			end if;
		end if;
	end process;
	
	process(phy_clk,reset_jtag)
	begin
		if reset_jtag = '1' then
			read_en <= '0';
		elsif phy_clk'event and phy_clk = '1' then
			if Control_reg = POP and update_ir = '1' then
				read_en <= '1';
			else 
				read_en <= '0';
			end if;
		end if;
	end process;

	

	process(phy_clk,reset_jtag)
	begin
		if reset_jtag = '1' then
			read_en_pulse <= '0';
			read_en_pulse_reg <= '0';
		elsif phy_clk'event and phy_clk = '1' then
			read_en_pulse <= read_en;
			read_en_pulse_reg <= read_en and not read_en_pulse;
		end if;
	end process;
	
--	innpll : pll port map (
--		areset => reset,
--		inclk0 => top_clk,
--		c0	    => cg_clk
--	);

	data_mux1 : data_mux2 port map (
		a => result_wire,
		b => push_in,
		sel => sel_data_mux,
		result => fifo_data
	);
	
	control_mux_1 : control_mux2 port map (
		a => write_jtag_fifo,
		b => write_en_pulse_reg,
		sel => sel_control_mux,
		result => write_req_fifo
	);
	
	control_mux_2 : control_mux2 port map (
		a => read_en_pulse_reg,
		b => c_wire8,
		sel => sel_control_mux2,
		result => read_req_fifo
	);

	top1 : top port map (
		inndata_fifo1 => d_wire1,
		inndata_fifo2 => d_wire1,
		data_b => q_temp,
		reset_inn => reset,
		clk => phy_clk, --cg_clk, --100MHz phy_clk,
		fifo_clk => phy_clk,--150MHZ
		start_inn => start,
		size => size1,
		wr_fifo1 => c_wire5,
		wr_fifo2 => c_wire3,
		wr_b => wr_b,
		--size_write => size_write_req,
		fifo1_full => c_wire6,
		fifo2_full => c_wire7,
		b_fifo_full => b_fifo_full,
		reset_full => reset_full,
		start_full => start_full,
		--size_full => size_full,
		result_fifo_empty => result_fifo_empty,
		result_fifo_read => result_fifo_read,
		done => done_cg,
		wrusedw1 => b_wire1,
		wrusedw2 => b_wire2,
		result => result_wire
	);
	
	FIFO_jtag : jtag_fifo PORT MAP (
		aclr => reset_jtag,
		data	 => fifo_data,
		rdclk	 => phy_clk,
		rdreq	 => read_req_fifo,
		wrclk	 => phy_clk,
		wrreq	 => write_req_fifo,
		q	 	 => q_temp,
		rdempty	 => rdempty_sig,
		wrfull	 => wrfull_sig,
		wrusedw	 => wrusedw_sig
	);
	
	virtual_jtag1 : virtual_jtag PORT MAP (
		ir_out	 => (others => '0'),
		tdo	 => tdo,
		ir_in	 => Control_reg,
		tck	 => tck,
		tdi	 => tdi,
		virtual_state_cdr	 => capture_dr,
		virtual_state_cir	 => virtual_state_cir_sig,
		virtual_state_e1dr	 => virtual_state_e1dr_sig,
		virtual_state_e2dr	 => virtual_state_e2dr_sig,
		virtual_state_pdr	 => virtual_state_pdr_sig,
		virtual_state_sdr	 => shift_dr,
		virtual_state_udr	 => update_dr,
		virtual_state_uir	 => update_ir
	);

with Control_reg select
		tdo <= push_in(0) when PUSH, push_out(0) when POP, tdi when others;
		
end rtl;