#include"Row.h"
#include<math.h>

template<class T>

class Matrix {

public:

	Matrix(int rows=0, int cols=0): matrix(NULL) {
		SetSize(rows,cols);
	}
	
	Matrix(const Matrix& m): matrix(NULL) {
		SetSize(m.numRows,m.numCols);
		for (int r=0;r<numRows;r++){
			matrix[r]=Row<T>(m.matrix[r]);
		}
	}

	void SetSize(int rows, int cols) {
		if (rows) delete[]matrix;
		if (cols > 0 && rows >0) {
			matrix=new Row<T>[rows];
			for (int i=0;i<rows;i++){
				matrix[i].SetRowSize(cols);
			}
		}
		else{ 
			rows=NULL;
		}
		numCols=cols;numRows=rows;
	}

	int GetCols() { return numCols;}
	int GetRows() { return numRows;}

	Row<T>& operator[](int index) {
		assert(index<numRows);
		return matrix[index];
	}

	Matrix& operator=(const Matrix& m) {
		SetSize(m.numRows,m.numCols);
		for (int r=0;r<numRows;r++){
			for (int c=0;c<numCols;c++){
				matrix[r][c]=(m.matrix[r][c]);
			}
		}
		return *this;
	} 

	Matrix operator+(Matrix& m) {
		assert(numCols==m.numCols && numRows==m.numRows);
		Matrix theMatrix(numRows,numCols);
		for (int r=0;r<numRows;r++)
		for (int c=0;c<numCols;c++)
			theMatrix[r][c]=matrix[r][c]+m.matrix[r][c];
		return theMatrix;
	}

	Matrix operator-(Matrix& m) {
		assert(numCols==m.numCols && numRows==m.numRows);
		Matrix theMatrix(numRows,numCols);
		for (int r=0;r<numRows;r++)
		for (int c=0;c<numCols;c++)
			theMatrix[r][c]=matrix[r][c]-m.matrix[r][c];
		return theMatrix;
	}

	Matrix& operator+=(const Matrix& m) {
		assert(numCols==m.numCols && numRows==m.numRows);
		for (int r=0;r<numRows;r++)
		for (int c=0;c<numCols;c++)
			matrix[r][c]+=m.matrix[r][c];
		return *this;
	}

	Matrix& operator-=( const Matrix& m) {
		assert(numCols==m.numCols && numRows==m.numRows);
		for (int r=0;r<numRows;r++)
		for (int c=0;c<numCols;c++)
			matrix[r][c]-=m.matrix[r][c];
		return *this;
	} 

	Matrix operator*(float s) {
		Matrix theMatrix(numRows,numCols);
		for (int r=0;r<numRows;r++)
		for (int c=0;c<numCols;c++)
			theMatrix[r][c]=matrix[r][c]*s;
		return theMatrix;
	} 

	Matrix operator*(Matrix& m) {
		assert(numCols==m.numRows);
		Matrix theMatrix(numRows,m.numCols);
		for (int r=0;r<numRows;r++) {
			for (int c=0;c<m.numCols;c++) {
				for (int i=0;i<numCols;i++) {
					theMatrix[r][c] += matrix[r][i]*m[i][c];
				}
			}
		}
		return theMatrix;
	}

	Matrix operator%(Matrix& m) {
		assert(numRows==m.numRows);
		Matrix theMatrix(numRows,m.numCols);
		for (int r=0;r<numRows;r++) {
			theMatrix[r][0] = matrix[r][0]*m[r][0];
		}
		return theMatrix;
	}

	//! = Transpose 
	Matrix operator!() {
		Matrix theMatrix(numCols,numRows);
		for (int i = 0; i < numCols; i++)
		{
			for (int j = 0; j < numRows; j++)
			{
				theMatrix[i][j] = matrix[j][i];
			}
		}
		return theMatrix;
	}
	Matrix operator-() {
		Matrix theMatrix(numRows, numCols);
		for (int i = 0; i < numCols; i++)
		{
			for (int j = 0; j < numRows; j++)
			{
				theMatrix[i][j] = -matrix[i][j];
			}
		}
		return theMatrix;
	}


	double toDouble(){
		assert(numCols == 1 && numRows == 1);
		double tall = (double) matrix[0][0];
		return tall;
	}

	Matrix operator/(Matrix& m) {
		assert(numCols == 1 && numRows == 1 && m.numCols == 1 && m.GetRows() == 1);
		Matrix theMatrix(1,1);
		theMatrix[0][0] = matrix[0][0]/m[0][0];
		return theMatrix;
	}

	//Finner lengden av en vektor
	double norm(){
		assert(numCols == 1);
		double tall = 0;
		for(int i = 0; i < numRows; i++){
			tall += pow(matrix[i][0], 2);
		}
		return sqrt(tall);
	}

	friend std::ostream& operator<< (std::ostream &os, Matrix& m){
	    //os << m.numRows << " " << m.numCols << " " << std::endl;
		for (int r=0;r<m.numRows;r++) {
		for (int c=0;c<m.numCols;c++) {
		os << m.matrix[r][c] << " ";
		}
		os << std::endl;
		}
	    return os;
	}

	friend std::istream& operator>> (std::istream& is, Matrix& m) {
		int rows,cols;
		is >> rows >> cols;
		m.SetSize(rows,cols);
		for (int r=0;r<rows;r++)
		for (int c=0;c<cols;c++)
			is >> m[r][c];
		return is;
	}
	
	Matrix ludecomp(){
		Matrix l(numCols,numRows);
		Matrix u(numCols,numRows);
		double sum;
		for(int i =0; i<numRows; i++){
			for(int j =0; j<numCols; j++){
				if(j==i){
					l[i][j] = 1;
				    //noe annet
				}
				if(j>i){
					l[i][j]=0;
					//noe annet
				}
				if(j<i){
					u[i][j]=0;
					//noe annet
				}
			}
		}
		//--------------------------------------------------------------------
	//AX=B
	//Decomposition of A into L and U
	int n=numCols;
	for(int i=0;i<n;i++)
	{
		for(int j=0;j<n;j++)
		{
			sum=0;
			if(i<=j)
			{
				for(int k=0;k<n;k++) 
					if(k!=i)
						sum=sum+l[i][k]*u[k][j];
				u[i][j]=matrix[i][j]-sum;
			}
			else 
			{
				for(int k=0;k<n;k++)
					if(k!=j)
						sum=sum+l[i][k]*u[k][j];
					l[i][j]=(matrix[i][j]-sum)/u[j][j];
			}
		}
	}
	return u;
	}
	
					


private:
	int numCols, numRows;
	Row<T>* matrix;
}; 



